# Python Libraries
from __future__ import print_function
from __future__ import division
import matplotlib.pyplot as plt
import matplotlib.ticker as mtick
import numpy

# Local Libraries
# Distal Libraries
from rgt.Util import Html
from rgt.CoverageSet import *
from rgt.ExperimentalMatrix import *
from shared_function import output_array, group_refque, color_groupded_region, multiple_correction, value2str
# Local test
dir = os.getcwd()
###########################################################################################
#                    Projection test
###########################################################################################


class Projection:
    def __init__(self, reference_path, query_path):
        # Reference
        self.rEM = ExperimentalMatrix()
        self.rEM.read(reference_path)
        self.rEM.remove_empty_regionset()
        self.references = self.rEM.get_regionsets()
        self.referencenames = self.rEM.get_regionsnames()
        # Query
        self.qEM = ExperimentalMatrix()
        self.qEM.read(query_path)
        self.qEM.remove_empty_regionset()
        self.query = self.qEM.get_regionsets()
        self.querynames = self.qEM.get_regionsnames()
        self.parameter = []
        self.background = None

    def group_refque(self, groupby=False):
        self.groupedreference, self.groupedquery = group_refque(self.rEM, self.qEM, groupby)

    def colors(self, colorby, definedinEM):
        ############# Color #####################################
        # self.color_list = colormap(self.qEM, colorby, definedinEM)
        self.color_list = color_groupded_region(self.qEM, self.groupedquery, colorby, definedinEM)
        # self.color_tags = gen_tags(self.qEM, colorby)
        # self.color_tags.append('Background')
        self.color_list['Background'] = '0.70'

    def ref_union(self):
        self.background = OrderedDict()
        for ty in self.groupedreference.keys():
            self.background[ty] = GenomicRegionSet("union of references")
            for r in self.groupedreference[ty]:
                self.background[ty].combine(r)
            self.background[ty].merge()

        for ty in self.groupedreference.keys():
            rlist = [ r.trim_by(background=self.background[ty]) for r in self.groupedreference[ty]]
            self.groupedreference[ty] = rlist
            qlist = [ q.trim_by(background=self.background[ty]) for q in self.groupedquery[ty]]
            self.groupedquery[ty] = qlist

    def set_background(self, bed_path):
        bg = GenomicRegionSet("background")
        bg.read_bed(bed_path)
        self.background = OrderedDict()
        for ty in self.groupedreference.keys():
            self.background[ty] = bg
            rlist = [ r.trim_by(background=bg) for r in self.groupedreference[ty]]
            self.groupedreference[ty] = rlist

            qlist = [ q.trim_by(background=bg) for q in self.groupedquery[ty]]
            self.groupedquery[ty] = qlist

    def projection_test(self, organism):
        self.bglist = OrderedDict()
        self.qlist = OrderedDict()
        self.plist = OrderedDict()
        self.interq_list = OrderedDict()
        self.lenlist = {}
        # print2(self.parameter, "\nProjection test")
        # print2(self.parameter, "{0:s}\t{1:s}\t{2:s}\t{3:s}\t{4:s}".format("Reference","Background", "Query", "Proportion", "p value"))

        all_p = {}
        for ty in self.groupedquery.keys():
            # print(ty)
            self.bglist[ty] = OrderedDict()
            self.qlist[ty] = OrderedDict()
            self.plist[ty] = OrderedDict()
            self.interq_list[ty] = OrderedDict()
            if self.background: bgset = self.background[ty]
            else: bgset = None

            for i, r in enumerate(self.groupedreference[ty]):
                # print(r.name)
                self.bglist[ty][r.name] = OrderedDict()
                self.qlist[ty][r.name] = OrderedDict()
                self.plist[ty][r.name] = OrderedDict()
                self.interq_list[ty][r.name] = OrderedDict()
                self.lenlist[r.name] = len(r)
                for j, q in enumerate(self.groupedquery[ty]):
                    # print(r.name, q.name, sep="\t")
                    if r.name == q.name: continue
                    else:
                        bg, ratio, p, interq = r.projection_test(q, organism, extra=True, background=bgset)
                        self.bglist[ty][r.name][q.name] = bg
                        self.qlist[ty][r.name][q.name] = ratio
                        self.plist[ty][r.name][q.name] = p
                        self.interq_list[ty][r.name][q.name] = interq
                        self.lenlist[q.name] = len(q)
                        # if r in self.backgrounds.keys(): pass
                        # else: self.backgrounds[r] = bg

        # multiple test correction
        multiple_correction(self.plist)

        for ty in self.groupedquery.keys():
            for i, r in enumerate(self.groupedreference[ty]):
                for j, q in enumerate(self.groupedquery[ty]):
                    # if r.name == q.name: continue
                    # else:
                    # bg = self.bglist[ty][r.name][q.name]
                    # ratio = self.qlist[ty][r.name][q.name]
                    # p = self.plist[ty][r.name][q.name]
                    self.qlist[ty][r.name]['Background'] = self.bglist[ty][r.name].values()[0]

    def output_interq(self, directory):
        """Output the intersected query to the reference in BED format"""
        try:
            os.stat(os.path.dirname(directory))
        except:
            os.mkdir(os.path.dirname(directory))
        try:
            os.stat(directory)
        except:
            os.mkdir(directory)
        for ty in self.interq_list.keys():
            if ty: g = ty+ "_"
            else:
                g = ""
            for r in self.interq_list[ty].keys():
                for q in self.interq_list[ty][r].keys():
                    self.interq_list[ty][r][q].write_bed(os.path.join(directory, g + q + "_intersected_" + r + ".bed"))

    def plot(self, logt=None, pw=3, ph=3):

        tw = pw
        th = len(self.qlist.keys()) * ph
        f, ax = plt.subplots(len(self.qlist.keys()), 1, dpi=300,figsize=(tw, th))

        # f, ax = plt.subplots(len(self.qlist.keys()),1)
        try:
            ax = ax.reshape(-1)
        except:
            ax = [ax]
        # nm = len(self.groupedreference.keys()) * len(self.groupedreference.values()[0]) * len(self.groupedquery.values()[0])
        # if nm > 40:
        #     f.set_size_inches(nm * 0.2 +1 ,7)

        g_label = []
        for ind_ty, ty in enumerate(self.qlist.keys()):
            g_label.append(ty)
            r_label = []
            for ind_r, r in enumerate(self.qlist[ty].keys()):
                r_label.append(r)
                width = 0.8 / (len(self.qlist[ty][r].keys()) + 1)  # Plus one background
                for ind_q, q in enumerate(self.qlist[ty][r].keys()):
                    x = ind_r + ind_q * width + 0.1
                    y = self.qlist[ty][r][q]
                    if y == 0 and logt: y = 0.000001
                    # print("    "+r+"     "+q+"     "+str(x)+"     "+str(y))
                    ax[ind_ty].bar(x, y, width=width, color=self.color_list[q], edgecolor="none",
                                   align='edge', log=logt, label=q)
            if logt:
                ax[ind_ty].set_yscale('log')
            else:
                ax[ind_ty].locator_params(axis='y', nbins=2)

            # ax[ind_ty].set_ylabel("Percentage of intersected regions",fontsize=12)
            ax[ind_ty].set_title(ty)
            ax[ind_ty].yaxis.tick_left()
            ax[ind_ty].set_ylabel('Percentage of intersected regions', fontsize=8)
            ax[ind_ty].set_xticks([i + 0.5 - 0.5 * width for i in range(len(r_label))])
            ax[ind_ty].set_xticklabels(r_label, rotation=30, ha="right", fontsize=8)
            ax[ind_ty].tick_params(axis='x', which='both', top='off', bottom='off', labelbottom='on')

            handles, labels = ax[ind_ty].get_legend_handles_labels()
            # uniq_labels = unique(labels)
            uniq_labels = [q.name for q in self.groupedquery[ty]] + ["Background"]

            ax[ind_ty].legend([handles[labels.index(l)] for l in uniq_labels], uniq_labels,
                              loc='center left', handlelength=1, handletextpad=1,
                              columnspacing=2, borderaxespad=0., prop={'size': 10}, bbox_to_anchor=(1.05, 0.5))
            for spine in ['top', 'right']:  # 'left', 'bottom'
                ax[ind_ty].spines[spine].set_visible(False)
        # f.text(-0.025, 0.5, "Percentage of intersected regions",fontsize=12, rotation="vertical", va="center")
        # f.tight_layout(pad=1.08, h_pad=None, w_pad=None)
        f.tight_layout()
        self.fig = f

    def heatmap(self):
        f, ax = plt.subplots(1, len(self.plist.keys()))
        try:
            ax = ax.reshape(-1)
        except:
            ax = [ax]

        g_label = []
        for ind_ty, ty in enumerate(self.plist.keys()):
            g_label.append(ty)
            r_label = []
            data = []
            for ind_r, r in enumerate(self.plist[ty].keys()):
                r_label.append(r)
                # data.append(self.plist[ty][r].values())
                for ind_q, q in enumerate(self.plist[ty][r].keys()):
                    pass
            da = numpy.array(data)
            da = da.transpose()
            # im = plt.imshow(da, cmap=ax[ind_r], vmin=, vmax, origin, extent, shape, filternorm, filterrad, imlim, resample, url, hold)

    def gen_html(self, directory, title, args, align=50):
        dir_name = os.path.basename(directory)
        statistic_table = []
        # check_dir(directory)
        html_header = "Projection Test: " + dir_name
        link_d = OrderedDict()
        link_d["Projection test"] = "index.html"
        link_d["Parameters"] = "parameters.html"

        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")
        html.add_figure("projection_test.png", align="center")

        header_list = ["No.",
                       "Reference<br>name",
                       "Query<br>name",
                       "Reference<br>number",
                       "Query<br>number",
                       "Proportion",
                       "Background<br>proportion",
                       "Positive<br>association<br>p-value",
                       "Negative<br>association<br>p-value"]
        statistic_table.append(["Reference_name", "Query_name", "Reference_number",
                                "Query_number", "Proportion", "Background_proportion",
                                "Positive_association_p-value", "Negative_association_p-value"])
        type_list = 'ssssssssssssssss'
        col_size_list = [5, 10, 10, 10, 10, 10, 10, 15, 15]

        nalist = []
        for ind_ty, ty in enumerate(self.plist.keys()):
            html.add_heading(ty, size=4, bold=False)
            data_table = []
            for ind_r, r in enumerate(self.plist[ty].keys()):
                rlen = str(self.lenlist[r])
                for ind_q, q in enumerate(self.plist[ty][r].keys()):
                    qlen = str(self.lenlist[q])
                    backv = value2str(self.qlist[ty][r]['Background'])
                    propor = value2str(self.qlist[ty][r][q])
                    pv = self.plist[ty][r][q]
                    if pv == "na":
                        nalist.append(r)
                        continue
                    elif self.qlist[ty][r][q] < args.cfp:
                        continue
                    else:
                        pvn = 1 - pv

                        if self.plist[ty][r][q] < 0.05:
                            if self.qlist[ty][r]['Background'] < self.qlist[ty][r][q]:
                                data_table.append([str(ind_ty), r, q, rlen, qlen, propor, backv,
                                                   "<font color=\"red\">" + value2str(pv) + "</font>", value2str(pvn)])
                                statistic_table.append([r, q, rlen, qlen, propor, backv, value2str(pv), value2str(pvn)])
                            else:
                                data_table.append([str(ind_ty), r, q, rlen, qlen, propor, backv,
                                                   value2str(pvn), "<font color=\"red\">" + value2str(pv) + "</font>"])
                                statistic_table.append([r, q, rlen, qlen, propor, backv, value2str(pvn), value2str(pv)])
                        else:
                            data_table.append(
                                [str(ind_ty), r, q, rlen, qlen, propor, backv, value2str(pv), value2str(pvn)])
                            statistic_table.append([r, q, rlen, qlen, propor, backv, value2str(pv), value2str(pvn)])

            html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, sortable=True)
            output_array(statistic_table, directory=directory, folder=title, filename="statistics" + ty + ".txt")

        header_list = ["Assumptions and hypothesis"]
        data_table = [['If the background proportion is too small, it may cause bias in p value.'],
                      [
                          'For projection test, the reference GenomicRegionSet should have non-zero length in order to calculate its background proportion.'],
                      ['P values are corrected by multiple test correction.'],
                      ['Positive association is defined by: Proportion > Background.'],
                      ['Negative association is defined by: Proportion < Background.']]

        nalist = set(nalist)
        if len(nalist) > 0:
            data_table.append([
                                  'The following references contain zero-length region which cause error in proportion calculation, please check it:<br>' +
                                  '     <font color=\"red\">' + ', '.join([s for s in nalist]) + '</font></p>'])
        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, cell_align="left")
        html.add_fixed_rank_sortable()

        html.write(os.path.join(directory, os.path.join(title, "index.html")))

        # Parameters
        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")
        header_list = ["Description", "Argument", "Value"]
        data_table = [["Reference", "-r", args.r],
                      ["Query", "-q", args.q],
                      ["Output directory", "-o", os.path.basename(args.o)],
                      ["Experiment title", "-t", args.t],
                      # ["Grouping tag", "-g", args.g],
                      # ["Coloring tag", "-c", args.c],
                      # ["Background", "-bg", args.bg],
                      ["Organism", "-organism", args.organism],
                      ["Cutoff of proportion", "-cfp", str(args.cfp)]]

        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, cell_align="left")
        html.add_free_content([
                                  '<a href="reference_experimental_matrix.txt" style="margin-left:100">See reference experimental matrix</a>'])
        html.add_free_content(
            ['<a href="query_experimental_matrix.txt" style="margin-left:100">See query experimental matrix</a>'])
        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See details</a>'])
        html.write(os.path.join(directory, os.path.join(title, "parameters.html")))

    def table(self, directory, folder):
        arr = numpy.array([["#reference", "query", "background", "proportion", "p-value"]])
        for ty in self.plist.keys():
            for r in self.plist[ty].keys():
                for q in self.plist[ty][r].keys():
                    ar = numpy.array(
                        [[r, q, self.qlist[ty][r]['Background'], self.qlist[ty][r][q], self.plist[ty][r][q]]])
                    arr = numpy.vstack((arr, ar))
        output_array(arr, directory, folder, filename="output_table.txt")

    def distribution(self, organism):
        genome = GenomicRegionSet("genome")
        genome.get_genome_data(organism)
        all_cov = genome.total_coverage()
        self.chrom_list = []
        for ss in genome:
            self.chrom_list.append(ss.chrom)
        self.chrom_list.sort()

        # self.distriDict = OrderedDict()
        self.disperDict = OrderedDict()

        for ty in self.groupedreference.keys():
            # self.distriDict[ty] = OrderedDict()
            self.disperDict[ty] = OrderedDict()
            # Reference
            for r in self.groupedreference[ty]:
                r.merge()
                len_r = r.total_coverage()
                # self.distriDict[ty][r.name] = []
                self.disperDict[ty][r.name] = []

                for ch in self.chrom_list:
                    rc = r.any_chrom(chrom=ch)
                    nr = sum([len(s) for s in rc])
                    # self.distriDict[ty][r.name].append(nr)
                    self.disperDict[ty][r.name].append(nr / len_r)

            # Query
            for q in self.groupedquery[ty]:
                q.merge()
                len_q = q.total_coverage()
                # self.distriDict[ty][q.name] = []
                self.disperDict[ty][q.name] = []

                for ch in self.chrom_list:
                    qc = q.any_chrom(chrom=chr)
                    nq = sum([len(s) for s in qc])
                    # self.distriDict[ty][q.name].append(nq)
                    self.disperDict[ty][q.name].append(nq / len_q)
            # Genome
            # self.distriDict[ty]["Genome"] = [len(genome.any_chrom(chrom=chr)) for chr in self.chrom_list]

            self.disperDict[ty]["Genome"] = [len(genome.any_chrom(chrom=chr)[0]) / all_cov for chr in self.chrom_list]

    def plot_distribution(self):
        def to_percentage(x, pos=0):
            return '{:.2f} %'.format(100 * x)

        self.fig = []

        for ty in self.disperDict.keys():
            colors = plt.cm.Set1(numpy.linspace(0.1, 0.9, len(self.disperDict[ty].keys()))).tolist()

            f, ax = plt.subplots()
            f.set_size_inches(10.5, 30)
            width = 0.9 / len(self.disperDict[ty].keys())
            ind = np.arange(len(self.chrom_list))
            coverage = self.disperDict[ty]

            for ind_r, r in enumerate(self.disperDict[ty].keys()):
                ax.barh(ind + width * ind_r, self.disperDict[ty][r], width, color=colors[ind_r])

            plt.xlabel('Percentage')
            ax.xaxis.set_major_formatter(mtick.FuncFormatter(to_percentage))

            ax.minorticks_off()
            ax.set_yticks([x + 0.5 for x in range(len(self.chrom_list))])
            ax.set_yticklabels(self.chrom_list, rotation=0, ha="right")
            ax.tick_params(axis='y', which='both', top='off', bottom='off', labelbottom='on')

            ax.legend(self.disperDict[ty].keys(), loc='center left', handlelength=1, handletextpad=1,
                      columnspacing=2, borderaxespad=0., prop={'size': 10}, bbox_to_anchor=(1.05, 0.5))
            for spine in ['top', 'right', 'left', 'bottom']:
                ax.spines[spine].set_visible(False)
            f.tight_layout(pad=1.08, h_pad=None, w_pad=None)
            self.fig.append(f)

    def gen_html_distribution(self, outputname, title, align=50):
        fp = os.path.join(dir, outputname, title)
        link_d = {title: "distribution.html"}
        html = Html(name="Viz", links_dict=link_d, fig_dir=os.path.join(dir, outputname, "fig"),
                    other_logo="viz", homepage="../index.html")
        for i, f in enumerate(self.fig):
            html.add_figure("distribution_test_" + str(i) + ".png", align="center")

        html.add_free_content(['<p style=\"margin-left: ' + str(align + 150) + '">' +
                               '** </p>'])

        type_list = 'ssssssssssssssssssssssssssssssssssssssssssssss'
        col_size_list = [10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
                         10, 10]
        data_table = []
        for ind_ty, ty in enumerate(self.disperDict.keys()):
            header_list = ["Chromosome"] + self.disperDict[ty].keys()
            html.add_heading(ty, size=4, bold=False)
            for i, ch in enumerate(self.chrom_list):
                # for ind_r,r in enumerate(self.disperDict[ty].keys()):

                data_table.append(
                    [ch] + ["{:.3f} %".format(100 * self.disperDict[ty][r][i]) for r in self.disperDict[ty].keys()])

        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align)

        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See parameters</a>'])
        html.add_free_content([
                                  '<a href="reference_experimental_matrix.txt" style="margin-left:100">See reference experimental matrix</a>'])
        html.add_free_content(
            ['<a href="query_experimental_matrix.txt" style="margin-left:100">See query experimental matrix</a>'])
        html.write(os.path.join(fp, "distribution.html"))



"""
GeneSet
===================
GeneSet describes genes and their expression.

"""

###############################################################################
# Libraries
###############################################################################
# Python
from __future__ import print_function
# Internal
from Util import GenomeData
# External

###############################################################################
# Class
###############################################################################

class GeneSet:
    """*Keyword arguments:*

        - name -- Name of the GeneSet
    """
    def __init__(self, name):
        self.name = name
        self.genes = [] #list of genes to consider
        self.values = {} #keys: gene, value: expression data as a list
        self.cond = []
        self.symbol_dict = {}

    def __len__(self):
        """Return the number of genes."""
        return len(self.genes)
    
    def __iter__(self):
        """Iterate this GeneSet."""
        return iter(self.genes)

    def add(self, gene_name, value=None):
        self.genes.append(gene_name)
        if value:
            self.values[gene_name] = value

    def read(self, geneListFile, score=False):
        """Read genes from the file.

        *Keyword arguments:*

            - geneListFile -- Path to the file which contains a list of genes.
        """
        with open(geneListFile) as f:
            for line in f:
                line = line.strip()
                if line:
                    l = line.split()
                    if l[0] != "":
                        if "." in l[0]:
                            # gene_name = l[0].upper()
                            gene_name = l[0].partition(".")[0].upper()
                            # self.genes.append(l[0].partition(".")[0].upper())
                        elif "," in l[0]:
                            gene_name = l[0].partition(",")[0].upper()
                        else:
                            gene_name = l[0].upper()
                        self.genes.append(gene_name)
                        if score:
                            self.values[gene_name] = l[1]
            
    def read_expression(self, geneListFile, header=False, valuestr=False):
        """Read gene expression data.

        *Keyword arguments:*
        
            - geneListFile -- Path to the file which contains genes and expression value.
            - header -- Read first line as header.
            - valuestr -- Keep the value as a string, otherwise convert to float number.
        """
        with open(geneListFile) as f:
            if header:
                l = f.readline()
                l = l.strip("\n")
                l = l.split("\t")
                self.cond = l[1:len(l)]
            else:
                l = f.readline()
                l = l.strip("\n")
                l = l.split("\t")
                self.cond = [str(e) for e in range(len(l)-1)]
            for line in f.readlines():
                line = line.strip("\n")
                l = line.split("\t")
                if l[0] != "":
                    try:
                        # print(l[0])
                        if "." in l[0] and l[0][0:2] == "EN":
                            na = l[0].partition(".")[0].upper()
                        else:
                            na = l[0].upper()
                        self.genes.append(na)
                        #self.values[l[0].upper()] = [float(v) for v in l[1:len(l)]]
                        if not valuestr:
                            self.values[na] = float(l[1])
                        else:
                            self.values[na] = l[1]
                    except:
                        print("*** error in loading gene: "+line)

    def get_all_genes(self, organism):
        """Get all gene names for a given organism.
        
        *Keyword arguments:*
        
            - organism -- Define the organism.
        """
        genome = GenomeData(organism=organism)
        self.genes = []
        f = open(genome.get_gene_regions())
        for l in f.readlines():
            l = l.strip("\n")
            l = l.split("\t")
            self.genes.append(l[3].upper())
        f.close()

        self.genes = list(set(self.genes))

    def subtract(self, gene_set):
        """Subtract another GeneSet.
        
        *Keyword arguments:*

            - gene_set -- Another GeneSet for subtracting with.
        """
        self.genes = [gene for gene in self.genes if gene not in gene_set.genes]

    def check(self, a_gene):
        """Check a gene is in the list or not

        *Keyword arguments:*

            - a_gene -- A gene symbol.
        """
        # g = a_gene.partition(".")[0].upper()
        g = a_gene.upper()

        return (g in self.genes)

    def save(self, filename):
        """Save gene list into the given filename."""
        if self.values:
            with open(filename, "w") as f:
                for g in self.genes:
                    print("\t".join([g, str(self.values[g])]), file=f)
        else:
            with open(filename, "w") as f:
                for g in self.genes:
                    print(g, file=f)


    # def load_alias(self, organism):
    #     """Load alias file for gene names"""
        
    #     genome_data = GenomeData(organism)
    #     # Opening alias file
    #     alias_file = open(genome_data.get_gene_alias(),"r")
    #     # Iterating over alias file entries
    #     for line in alias_file:
    #         ll = line.strip().split("\t")
    #         ensembl_id = ll[0]
    #         official_name = ll[1]
    #         alias_vec = ll[2].split("&")
    #         self.symbol_dict[ensembl_id] = official_name
    #         for e in alias_vec:
    #             try: self.alias_dict[e].append(ensembl_id)
    #             except Exception: self.alias_dict[e] = [ensembl_id]
    #     # Termination
    #     alias_file.close()

    # def ensembl2symbol(self, organism):
    #     """Convert ensembl ID to gene symbol"""
    #     if not self.symbol_dict:
    #         self.load_alias()
    #     curr_list = self.genes
    #     mapped_list = []
    #     unmapped_list = []
    #     for e in curr_list:
    #         try: mapped_list.append(self.symbol_dict[e])
    #         except Exception: unmapped_list.append(self.symbol_dict[e])

    #     self.genes = mapped_list
        
        


# Python Libraries
from __future__ import print_function
from __future__ import division
from shared_function import *


# Local test
dir = os.getcwd()

###########################################################################################
#                    Venn Diagram
###########################################################################################

class Venn:
    def __init__(self, sets, organism):
        self.sets = sets
        self.num = len(sets)
        self.organism = organism
        self.process_genesets()

    def process_genesets(self):
        self.gene_sets = []
        for gs in self.sets:
            if gs.endswith(".bed"):
                gregions = GenomicRegionSet(gs)
                gregions.read_bed(gs)
                associated_gr = gregions.gene_association(organism=self.organism, 
                    promoterLength=1000, threshDist=0, show_dis=False)
                genes = []
                for r in associated_gr: 
                    if ":" in r.name:
                        rgg = r.name.upper().split(":")
                        genes += rgg
                    else: genes.append(r.name.upper())
                self.gene_sets.append(set(genes))
                
            elif gs.endswith(".txt"):
                genes = []
                with open(gs) as f:
                    for line in f:
                        g = line.strip().split()
                        genes.append(g[0].upper())
                self.gene_sets.append(set(genes))

    def venn_diagram(self, directory, title, labels):
        
        if len(self.sets) == 2: 
            from matplotlib_venn import venn2

            f = plt.figure(figsize=(10,10))
            inter = len(self.gene_sets[0].intersection(self.gene_sets[1]))
            fig_venn = venn2(subsets = (len(self.gene_sets[0]) - inter, inter, len(self.gene_sets[1]) - inter),
                             set_labels = (self.sets[0].partition("/")[2].partition(".")[0], 
                                           self.sets[1].partition("/")[2].partition(".")[0]))
            plt.title("Sample Venn diagram")
            return f

        elif len(self.sets) == 3:
            from matplotlib_venn import venn3
            def write_genes(filename, geneset):
                with open(filename, "w") as g:
                    for gene in geneset: print(gene, file=g)

            f = plt.figure(figsize=(10,10))

            s100 = self.gene_sets[0] - self.gene_sets[1] - self.gene_sets[2]
            write_genes(filename=os.path.join(directory, title,"list_"+labels[0]+".txt"),
                        geneset=s100)
            s010 = self.gene_sets[1] - self.gene_sets[0] - self.gene_sets[2]
            write_genes(filename=os.path.join(directory, title,"list_"+labels[1]+".txt"),
                        geneset=s010)
            s001 = self.gene_sets[2] - self.gene_sets[0] - self.gene_sets[1]
            write_genes(filename=os.path.join(directory, title,"list_"+labels[2]+".txt"),
                        geneset=s001)
            s111 = self.gene_sets[0].intersection(self.gene_sets[1].intersection(self.gene_sets[2]))
            write_genes(filename=os.path.join(directory, title,"list_"+labels[0]+"_"+labels[1]+"_"+labels[2]+".txt"),
                        geneset=s111)
            s110 = self.gene_sets[0].intersection(self.gene_sets[1]) - self.gene_sets[2]
            write_genes(filename=os.path.join(directory, title,"list_"+labels[0]+"_"+labels[1]+".txt"),
                        geneset=s110)
            s011 = self.gene_sets[1].intersection(self.gene_sets[2]) - self.gene_sets[0]
            write_genes(filename=os.path.join(directory, title,"list_"+labels[1]+"_"+labels[2]+".txt"),
                        geneset=s011)
            s101 = self.gene_sets[0].intersection(self.gene_sets[2]) - self.gene_sets[1]
            write_genes(filename=os.path.join(directory, title,"list_"+labels[0]+"_"+labels[2]+".txt"),
                        geneset=s101)
            
            fig_venn = venn3(subsets = (len(s100),len(s010),len(s110),len(s001),len(s101),len(s011),len(s111)),
                             set_labels = labels)
            return f


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles associated
with two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
from rgt.GenomicRegion import GenomicRegion
from rgt.GenomicRegionSet import GenomicRegionSet
import sys
# import re
from scipy.stats.mstats import zscore
from rgt.motifanalysis.Statistics import multiple_test_correction
import numpy as np
from numpy import log10


def merge_data(regions):
    for el in regions:
        tmp = el.data
        d = tmp.split('_$_')
        c1, c2 = 0, 0
        logpvalue = []
        for tmp in d:
            tmp = tmp.split(',')
            c1 += int(tmp[0].replace("(", ""))
            c2 += int(tmp[1])
            logpvalue.append(float(tmp[2].replace(")", "")))

        el.data = str((c1, c2, max(logpvalue)))


def output(name, regions):
    color = {'+': '255,0,0', '-': '0,255,0'}
    output = []
    
    for i, el in enumerate(regions):
        tmp = el.data.split(',')
        #counts = ",".join(map(lambda x: re.sub("\D", "", x), tmp[:len(tmp)-1]))
        main_sep = ':' #sep <counts> main_sep <counts> main_sep <pvalue>
        int_sep = ';' #sep counts in <counts>
        counts = ",".join(tmp).replace('], [', ';').replace('], ', int_sep).replace('([', '').replace(')', '').replace(', ', main_sep)
        pvalue = float(tmp[len(tmp)-1].replace(")", "").strip())
        
        output.append("\t".join([el.chrom, el.initial, el.final, 'Peak'+str(i), 1000, el.orientation, el.initial, el.final, \
              color[el.orientation], 0, counts]), pvalue)
    
    return output

def merge_delete(ext_size, merge, peak_list, pvalue_list):
#     peaks_gain = read_diffpeaks(path)
    
    regions_plus = GenomicRegionSet('regions') #pot. mergeable
    regions_minus = GenomicRegionSet('regions') #pot. mergeable
    regions_unmergable = GenomicRegionSet('regions')
    last_orientation = ""
    
    for i, t in enumerate(peak_list):
        chrom, start, end, c1, c2, strand, ratio = t[0], t[1], t[2], t[3], t[4], t[5], t[6]
        r = GenomicRegion(chrom = chrom, initial = start, final = end, name = '', \
                          orientation = strand, data = str((c1, c2, pvalue_list[i], ratio)))
        if end - start > ext_size:
            if strand == '+':
                if last_orientation == '+':
                    region_plus.add(r)
                else:
                    regions_unmergable.add(r)
            elif strand == '-':
                if last_orientation == '-':
                    region_mins.add(r)
                else:
                    regions_unmergable.add(r)
                    
                    
    if merge:
        regions_plus.extend(ext_size/2, ext_size/2)
        regions_plus.merge()
        regions_plus.extend(-ext_size/2, -ext_size/2)
        merge_data(regions_plus)
        
        regions_minus.extend(ext_size/2, ext_size/2)
        regions_minus.merge()
        regions_minus.extend(-ext_size/2, -ext_size/2)
        merge_data(regions_minus)
    
    results = GenomicRegionSet('regions')
    for el in regions_plus:
        results.add(el)
    for el in regions_minus:
        results.add(el)
    for el in regions_unmergable:
        results.add(el)
    results.sort()
    
    return results

def filter_by_pvalue_strand_lag(ratios, pcutoff, pvalues, output, no_correction, name, singlestrand):
    """Filter DPs by strang lag and pvalue"""
    if not singlestrand:
        zscore_ratios = zscore(ratios)
        ratios_pass = np.where(np.bitwise_and(zscore_ratios > -2, zscore_ratios < 2) == True, True, False)
    if not no_correction:
        pv_pass = [True] * len(pvalues)
        pvalues = map(lambda x: 10**-x, pvalues)
        
        _output_BED(name + '-uncor', output, pvalues, pv_pass)
        _output_narrowPeak(name + '-uncor', output, pvalues, pv_pass)
        
        pv_pass, pvalues = multiple_test_correction(pvalues, alpha=pcutoff)
    else:
        pv_pass = np.where(np.asarray(pvalues) >= -log10(pcutoff), True, False)
    
    if not singlestrand:
        filter_pass = np.bitwise_and(ratios_pass, pv_pass)
        assert len(pv_pass) == len(ratios_pass)
    else:
        filter_pass = pv_pass
    
    assert len(output) == len(pvalues)
    assert len(filter_pass) == len(pvalues)
    
    return output, pvalues, filter_pass

def _output_BED(name, output, pvalues, filter):
    f = open(name + '-diffpeaks.bed', 'w')
     
    colors = {'+': '255,0,0', '-': '0,255,0'}
    bedscore = 1000
    
    for i in range(len(pvalues)):
        c, s, e, strand, counts = output[i]
        p_tmp = -log10(pvalues[i]) if pvalues[i] > 0 else sys.maxint
        counts = ';'.join(counts.split(';')[:2] + [str(p_tmp)])
        
        if filter[i]:
            print(c, s, e, 'Peak' + str(i), bedscore, strand, s, e, colors[strand], 0, counts, sep='\t', file=f)
    
    f.close()

def _output_narrowPeak(name, output, pvalues, filter):
    """Output in narrowPeak format,
    see http://genome.ucsc.edu/FAQ/FAQformat.html#format12"""
    f = open(name + '-diffpeaks.narrowPeak', 'w')
    for i in range(len(pvalues)):
        c, s, e, strand, _ = output[i]
        p_tmp = -log10(pvalues[i]) if pvalues[i] > 0 else sys.maxint
        if filter[i]:
            print(c, s, e, 'Peak' + str(i), 0, strand, 0, p_tmp, 0, -1, sep='\t', file=f)
    f.close()
    
def filter_deadzones(bed_deadzones, peak_regions):
    """Filter by peaklist by deadzones"""
    deadzones = GenomicRegionSet('deadzones')
    deadzones.read_bed(bed_deadzones)
    peak_regions = peak_regions.subtract(deadzones, whole_region=True)
    
    return peak_regions
    
if __name__ == '__main__':
    ext_size1 = int(sys.argv[1]) #100
    ext_size2 = int(sys.argv[2]) #100
    path = sys.argv[3] # '/home/manuel/merge_test.data'
    merge = sys.argv[4] #True #for histones
    
    regions = merge_delete(path, ext_size1, '+', merge)
    #regions_minus = merge_delete(path, ext_size2, '-', merge)
    
    i = 0
    i = output(regions, i)
    #i = output(regions_minus, i, '-')
    
    



# Python Libraries
from __future__ import print_function
from __future__ import division

# Local Libraries
# Distal Libraries
from shared_function import *

# Local test
dir = os.getcwd()
###########################################################################################
#                    Jaccard test
###########################################################################################

class Jaccard:
    def __init__(self, reference_path, query_path):
        self.rEM, self.qEM = ExperimentalMatrix(), ExperimentalMatrix()
        self.rEM.read(reference_path)
        self.qEM.read(query_path)
        self.references = self.rEM.get_regionsets()
        self.referencenames = self.rEM.get_regionsnames()
        self.query = self.qEM.get_regionsets()
        self.querynames = self.qEM.get_regionsnames()
        self.parameter = []

    def group_refque(self, groupby=False):
        self.groupedreference, self.groupedquery = group_refque(self.rEM, self.qEM, groupby)

    def colors(self, colorby, definedinEM):
        self.color_list = color_groupded_region(self.qEM, self.groupedquery, colorby, definedinEM)
        # self.color_list['Background'] = '0.70'

    def jaccard_test(self, runtime, organism):
        self.jlist = OrderedDict()
        self.realj = OrderedDict()
        self.plist = OrderedDict()
        self.rlen = {}
        self.qlen = {}
        self.rt = runtime
        self.nalist = []
        print2(self.parameter, "\nJaccard Test")
        print2(self.parameter,
               "{0:s}\t{1:s}\t{2:s}\t{3:s}\t{4:s}\t{5:s}".format("Reference", "Query", "Repeats", "True_Jaccard_index",
                                                                 "p-value", "Time"))
        for ty in self.groupedreference.keys():
            self.jlist[ty] = OrderedDict()
            self.realj[ty] = OrderedDict()
            self.plist[ty] = OrderedDict()
            for i, r in enumerate(self.groupedreference[ty]):
                if r.total_coverage() == 0 and len(r) > 0:
                    self.nalist.append(r.name)
                    continue
                else:
                    if r.name not in self.rlen.keys(): self.rlen[r.name] = len(r)
                    self.jlist[ty][r.name] = OrderedDict()
                    self.realj[ty][r.name] = OrderedDict()
                    self.plist[ty][r.name] = OrderedDict()
                    for j, q in enumerate(self.groupedquery[ty]):
                        ts = time.time()
                        # print(q.name + "      " + str(len(q.sepuences[0])))

                        # The real jaccard index from r and q
                        if q.total_coverage() == 0 and len(q) > 0:
                            self.nalist.append(q.name)
                            continue
                        else:
                            if q.name not in self.qlen.keys(): self.qlen[q.name] = len(q)
                            self.jlist[ty][r.name][q.name] = []
                            self.realj[ty][r.name][q.name] = q.jaccard(r)
                            for k in range(runtime):
                                random = q.random_regions(organism=organism, multiply_factor=1, overlap_result=True,
                                                          overlap_input=True, chrom_M=False)
                                self.jlist[ty][r.name][q.name].append(r.jaccard(random))
                            # How many randomizations have higher jaccard index than the real index?
                            p = len([x for x in self.jlist[ty][r.name][q.name] if
                                     x > self.realj[ty][r.name][q.name]]) / runtime
                            self.plist[ty][r.name][q.name] = p
                            te = time.time()
                            print2(self.parameter, r.name + "\t" + q.name + "\tx" + str(runtime) + "\t" +
                                   value2str(self.realj[ty][r.name][q.name]) + "\t" + value2str(p) + "\t" +
                                   str(datetime.timedelta(seconds=round(te - ts))))

    def plot(self, logT=False, pw=3, ph=3):
        """ Return boxplot from the given tables.

        """
        self.fig = []
        self.xtickrotation, self.xtickalign = 0, "center"

        tw = pw
        # th = len(self.jlist.keys()) * ph
        # f, ax = plt.subplots(len(self.jlist[t].keys()), 1, dpi=300,
        #                       figsize=(tw, th) )
        legend_x = 1.05
        for it, t in enumerate(self.jlist.keys()):
            # f, axarr = plt.subplots(1, len(self.jlist[t].keys()), dpi=300, sharey=True)
            # legend_x = 1.05
            # nm = len(self.jlist.keys()) * len(self.jlist.values()[0]) * len(self.jlist.values()[0])
            # if nm > 30:
            #     f.set_size_inches(nm * 0.1 +1 ,nm * 0.1 +1)
            #     legend_x = 1.2
            #     self.xtickrotation, self.xtickalign = 70,"right"
            th = len(self.jlist[t].keys()) * ph
            f, axarr = plt.subplots(len(self.jlist[t].keys()), 1, dpi=300,
                                    figsize=(tw, th))
            try:
                axarr = axarr.reshape(-1)
            except:
                axarr = [axarr]
            plt.subplots_adjust(bottom=0.3)
            if logT:
                axarr[0].set_ylabel("Jaccard index (log)", fontsize=8)
            else:
                axarr[0].set_ylabel("Jaccard index (Intersect/Union)", fontsize=8)

            for i, r in enumerate(self.jlist[t].keys()):
                # axarr[i].set_title(r, y=0.94)
                if logT:
                    axarr[i].set_yscale('log')
                else:
                    axarr[i].locator_params(axis='y', nbins=4)
                axarr[i].tick_params(axis='y', direction='out')

                axarr[i].yaxis.tick_left()
                axarr[i].yaxis.grid(True, linestyle='-', which='major', color='lightgrey', alpha=0.7, zorder=1)
                d = []  # Store data within group
                color_t = []  # Store tag for coloring boxes
                # x_ticklabels = []  # Store ticklabels
                axarr[i].set_xlabel(r, rotation=self.xtickrotation, ha=self.xtickalign)

                for j, q in enumerate(self.jlist[t][r].keys()):
                    d.append(self.jlist[t][r][q])
                    color_t.append(self.color_list[q])
                    # x_ticklabels.append(q)
                # Fine tuning boxplot


                bp = axarr[i].boxplot(d, notch=False, sym='o', vert=True, whis=1.5, positions=None, widths=None,
                                      patch_artist=True, bootstrap=None)
                z = 10  # zorder for bosplot
                plt.setp(bp['whiskers'], color='black', linestyle='-', linewidth=0.8, zorder=z)
                plt.setp(bp['fliers'], markerfacecolor='gray', color='white', alpha=0.3, markersize=1.8, zorder=z)
                plt.setp(bp['caps'], color='white', zorder=z)
                plt.setp(bp['medians'], color='black', linewidth=0.5, zorder=z + 1)
                legends = []
                for patch, color in zip(bp['boxes'], color_t):
                    patch.set_facecolor(color)  # When missing the data, the color patch will exceeds
                    patch.set_edgecolor("none")
                    patch.set_zorder(z)
                    legends.append(patch)
                axarr[i].scatter(x=range(1, 1 + len(self.jlist[t][r].keys())),
                                 y=[y for y in self.realj[t][r].values()],
                                 s=10, c="red", edgecolors='none', zorder=2)
                # Fine tuning subplot
                # axarr[i].set_xticks(range(len(self.jlist[t][r].keys())))
                # plt.xticks(xlocations, sort_tags, rotation=90, fontsize=10)
                # axarr[i].set_xticklabels(self.jlist[t][r].keys(), rotation=0, fontsize=10)

                # axarr[i].set_ylim(bottom=0.95)
                for spine in ['top', 'right', 'left', 'bottom']:
                    axarr[i].spines[spine].set_visible(False)
                axarr[i].tick_params(axis='x', which='both', bottom='off', top='off', labelbottom='off')

                if i > 0:
                    plt.setp(axarr[i].get_yticklabels(), visible=False)
                    # plt.setp(axarr[i].get_yticks(),visible=False)
                    axarr[i].minorticks_off()
                    axarr[i].tick_params(axis='y', which='both', left='off', right='off', labelbottom='off')

            plt.setp([a.get_yticklabels() for a in axarr[1:]], visible=False)
            axarr[-1].legend(legends[0:len(self.jlist[t][r].keys())], self.jlist[t][r].keys(), loc='center left',
                             handlelength=1,
                             handletextpad=1, columnspacing=2, borderaxespad=0., prop={'size': 10},
                             bbox_to_anchor=(legend_x, 0.5))
            # f.tight_layout(pad=2, h_pad=None, w_pad=None)
            self.fig.append(f)

    def gen_html(self, outputname, title, align=50):
        fp = os.path.join(dir, outputname, title)
        link_d = {title: "index.html"}
        html = Html(name="Viz", links_dict=link_d,
                    fig_dir=os.path.join(dir, outputname, "fig"), other_logo="viz",
                    homepage="../index.html")
        for i in range(len(self.fig)):
            html.add_figure("jaccard_test" + str(i + 1) + ".png", align="center")

        header_list = ["Reference<br>name",
                       "Query<br>name",
                       "Reference<br>number",
                       "Query<br>number",
                       "True<br>Jaccard<br>index",
                       "Average<br>random<br>Jaccard",
                       "Positive<br>Association<br>p-value",
                       "Negative<br>Association<br>p-value"]

        type_list = 'sssssssssssss'
        col_size_list = [10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10]
        data_table = []

        for ind_ty, ty in enumerate(self.jlist.keys()):
            # html.add_heading(ty, size = 4, bold = False)
            for ind_r, r in enumerate(self.jlist[ty].keys()):
                for ind_q, q in enumerate(self.jlist[ty][r].keys()):
                    rej = self.realj[ty][r][q]
                    rj = numpy.mean(self.jlist[ty][r][q])
                    p = self.plist[ty][r][q]
                    np = 1 - p
                    rl = str(self.rlen[r])
                    ql = str(self.qlen[q])

                    if self.plist[ty][r][q] < 0.05:
                        if self.realj[ty][r][q] > rj:
                            data_table.append([r, q, rl, ql, value2str(rej), value2str(rj),
                                               "<font color=\"red\">" + value2str(p) + "</font>",
                                               value2str(np)])
                        else:
                            data_table.append([r, q, rl, ql, value2str(rej), value2str(rj),
                                               value2str(np),
                                               "<font color=\"red\">" + value2str(p) + "</font>"])
                    else:
                        data_table.append([r, q, rl, ql, value2str(rej), value2str(rj), value2str(p), value2str(np)])

        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align)

        header_list = ["Assumptions and hypothesis"]
        data_table = [['Randomization was performed ' + str(self.rt) + ' times.'],
                      [
                          'For projection test, the reference and query should have non-zero length in order to calculate its Jaccard index.'],
                      ['Positive association is defined by: True Jaccard index > Averaged random Jaccard.'],
                      ['Negative association is defined by: True Jaccard index < Averaged random Jaccard.']]
        self.nalist = set(self.nalist)
        if len(self.nalist) > 0:
            data_table.append([
                                  'The following region sets contain zero-length regions which cause error in Jaccard index calculation, please check it:<br>' +
                                  '<font color=\"red\">' + ', '.join([s for s in self.nalist]) + '</font>'])
        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, cell_align="left")

        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See parameters</a>'])
        html.add_free_content([
                                  '<a href="reference_experimental_matrix.txt" style="margin-left:100">See reference experimental matrix</a>'])
        html.add_free_content(
            ['<a href="query_experimental_matrix.txt" style="margin-left:100">See query experimental matrix</a>'])
        html.write(os.path.join(fp, "index.html"))

    def table(self, directory, folder):
        arr = numpy.array([["#reference", "query", "true_jaccard", "random_jaccard", "p-value"]])
        for ty in self.plist.keys():
            for r in self.plist[ty].keys():
                for q in self.plist[ty][r].keys():
                    ar = numpy.array([[r, q, self.realj[ty][r][q], self.qlist[ty][r][q], self.plist[ty][r][q]]])
                    arr = numpy.vstack((arr, ar))
        output_array(arr, directory, folder, filename="output_table.txt")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
MotifSet
===================
Represents a transcription factor motif and the standardization of motif annotation.

"""

# Python
from __future__ import print_function
import os
import sys
import sets
import glob
import sets
from copy import deepcopy

# Internal
from GenomicRegion import GenomicRegion
from Util import GenomeData, MotifData, AuxiliaryFunctions

#TODO:
#- connect motif with file and biopython object
#- keep track of databases (???)

class Motif:
    """Represents transcription factor motifs.

    *Keyword arguments:*

      - tf_id -- Transcription factor ID.
      - name -- Transcription factor name (symbol).
      - database -- Database/repository in which this motif was obtained from.
      - tf_class -- Class of transcription factor motif.
      - genes -- Genes in which transcription factor binds to.
      - genes_suffix -- Gene suffixes.
    """

    def __init__(self, tf_id, name, database, tf_class, genes, genes_suffix):
        self.id = tf_id
        self.name = name
        self.database = database
        self.tf_class=tf_class
        self.genes = genes
        self.genes_suffix = genes_suffix

class MotifSet:
    """Represents a set of motifs."""

    def __init__(self):
        self.motifs_map={}
        self.genes_map={}
        self.genes_suffix_map={}
        self.networks={}
        self.motifs_enrichment={}
        self.conditions=[]
        

    def add(self, new_motif):
        """Adds a new motif to this set.

        *Keyword arguments:*

          - new_motif -- New motif to be added.
        """

        #new_motif=Motif(tf_id, name, database, tf_class, genes, genes_suffix)
        try:
            self.motifs_map[new_motif.name]
        except:
            self.motifs_map[new_motif.name]=new_motif
            for g in new_motif.genes:
              g=g.upper()
              try:
                motifs_aux=self.genes_map[g]
                motifs_aux.append(new_motif)
              except:  
                self.genes_map[g]=[new_motif]
            for g in new_motif.genes_suffix:
              try:
                motifs_aux=self.genes_suffix_map[g]
                motifs_aux.append(new_motif)
              except:  
                self.genes_suffix_map[g]=[new_motif]

    def match_suffix(self, gene_name):
        """Match with gene suffix

        *Keyword arguments:*

          - gene_name -- Gene name to perform the match.

        *Return:*

          - res -- ID of mapped genes.
        """

        res=[]
        gene_name=gene_name.upper()
        for s in self.genes_suffix_map.keys():
          if gene_name.startswith(s):
            res.append(s)
        return res   

    def filter_by_motifs(self, motifs):
        """Filter this motif set by defined motifs.

        *Keyword arguments:*

          - motifs -- Motifs in which to filter this set.

        *Return:*

          - motif_sets -- Filtered motif sets.
        """

        motif_sets=MotifSet()
        for m in motifs:
          try:
            motif_sets.add(self.motifs_map[m])
          except Exception:
            print("motif not found: "+str(m))
        return motif_sets


    def filter_by_genes(self, genes, search="exact"):
        """This method returns motifs associated to genes. The search has three modes: 
            1. 'exact' - exact match only
            2. 'inexact' - genes with no exact match are searched for inexact matcth
            3. 'all' - all genes are applied to an inexact match

        *Keyword arguments:*

          - genes -- Gene set to perform the filtering.
          - search -- Search mode (default = 'exact').

        *Return:*

          - motif_sets -- Filtered motif sets.
          - genes_motifs -- Dictionary of genes to motifs.
          - motifs_genes -- Dictionary of motifs to genes.
        """
        motif_sets=MotifSet()
        motifs_genes={}
        genes_motifs={}
        not_found_genes= [] # keep genes for inexact search
        genes=genes.genes
        for g in genes:
           g=g.upper()
           try:
             motifs=self.genes_map[g]
             for m in motifs:
                motif_sets.add(m)
                try:
                  genes_motifs[g].append(m.name)   
                except:
                  genes_motifs[g]=[m.name]
                try:
                  motifs_genes[m.name].append(g)   
                except:
                  motifs_genes[m.name]=[g]

           except Exception, e:
             not_found_genes.append(g) # keep genes for inexact search 
        if search == "inexact":
            genes = not_found_genes;
        elif search == "exact":
            genes=[];                  
        for g in genes:
             suffs=self.match_suffix(g)
             for s in suffs:
               motifs=self.genes_suffix_map[s]
               for m in motifs:
                  motif_sets.add(m)
                  try:
                    genes_motifs[g].append(m.name)   
                  except:
                    genes_motifs[g]=[m.name]
                  try:
                    motifs_genes[m.name].append(g)   
                  except:
                    motifs_genes[m.name]=[g]
        return motif_sets,genes_motifs,motifs_genes
                     

    def read_file(self, file_name_list):
        """Reads TF annotation in mtf (internal format; check manual) format.
        
        *Keyword arguments:*

          - file_name_list -- A list with .mtf files.
        """

        # Iterating over the file name list
        for file_name in file_name_list:

            # Opening MTF file
            #try: 
            mtf_file = open(file_name,"r")
            #except Exception: pass # TODO

            # Reading file
            for line in mtf_file:
                # Processing line
                line_list = line.strip().split("\t")
                tf_id=line_list[0]
                name=line_list[1]
                database=line_list[2]
                tf_class=int(line_list[3])
                genes=line_list[4].split(";")
                genes_suffix=line_list[5].split(";")

                self.add(Motif(tf_id, name, database, tf_class, genes, genes_suffix))


            # Termination
            mtf_file.close()


    def  read_motif_targets_enrichment(self, enrichment_files, pvalue_threshold):
        """Reads current output of motif enrichment analysis to get gene targets.

        *Keyword arguments:*

          - enrichment_files -- Enrichment files to read.
          - pvalue_threshold -- P-value threshold for motif acceptance.
        """
        # reading networks
        for f in glob.glob(enrichment_files): 
          # use last dir name as name for condition
          condition=os.path.dirname(f)
          condition=condition.split("/")[-1]
          self.conditions.append(condition)
          network={}
          for line in open(f):
            line=line.strip("\n")
            values=line.split("\t")
            motif=values[0]
            try:
                self.motifs_map[motif]  
                p_value=float(values[2])
                genes=values[9].split(",")
                if (pvalue_threshold>=p_value):
                    network[motif]=genes
                try:
                  p_values=self.motifs_enrichment[motif]
                  p_values[condition]=p_value
                except:
                  p_values={}                   
                  p_values[condition]=p_value
                  self.motifs_enrichment[motif]=p_values                     
            except Exception, e:
                print("motif not found: "+motif+str(e))

          self.networks[condition]=network


    def write_enrichment_table(self, threshold, out_file, motifs_map):
        """Writes enrichment table for network generation.

        *Keyword arguments:*

          - threshold -- P-value threshold for motif acceptance.
          - out_file -- Output file name.
          - motifs_map -- Mapping of motifs.
        """

        f = open(out_file,"w")
        f.write("\t"+("\t".join(self.conditions))+"\n")
        motifs=motifs_map.keys()       
        for v in self.motifs_enrichment.keys():
            values=self.motifs_enrichment[v]
            filter_p=False
            p_values=[]
            for c in self.conditions:
                try:
                    pvalue=values[c]
                    p_values.append(str(pvalue))    
                    if pvalue<=threshold:
                        filter_p=True
                except:
                    p_values.append("1")
            if ((filter_p) & (v in motifs)):
                genes="|".join(motifs_map[v])
                f.write(v+"|"+genes+"\t"+("\t".join(p_values))+"\n")


#    def read_motif_targets(self,file_name,condition,pvalue_threshold):
#        ''' reads current output of motif enrichment analysis to get gene targets
#            XXX - This function could be made internal by integration with motif enrichment search
#        '''
#        network={}
#        for line in open(file_name):
#            line=line.strip("\n")
#            values=line.split("\t")
#            motif=values[0]
#            try:
#                self.motifs_map[motif]  
#                pvalue=float(values[2])
#                genes=values[9].split(",")
#                if (pvalue_threshold>=pvalue):
#                    network[motif]=genes
#            except Exception, e:
#                print("motif not found: "+motif+str(e))
#        self.networks[condition]=network

#    def get_motifs_networks():
#        '''
#        Getting all motifs presents in at least one network
#        '''
#        motifs={}
#        for net in self.networks:
#            for m in net.keys():
#              motifs[m]=[]
#        return motifs.keys()

    def write_cytoscape_network(self, genes, gene_mapping_search, out_path, targets, threshold):
        """Write files to be used as input for cytoscape. It recieves a list of genes to map to, a mapping search strategy and path for outputting files.

        *Keyword arguments:*

          - genes -- Gene set.
          - gene_mapping_search -- Gene mapping.
          - out_path -- Output path.
          - targets -- Gene targets.
          - threshold -- Threshold for motif acceptance.
        """
        #getting mapping of genes to motifs
        [motif_sets,genes_motifs,motifs_genes]=self.filter_by_genes(genes,gene_mapping_search)

        f=open(out_path+"/mapping_tf_genes.txt","w")
        motifs_all={}
        for gene in genes_motifs.keys():
          motifs = genes_motifs[gene]
          for m in motifs:
            try:
              g=motifs_all[m]
              if gene not in g:
                motifs_all[m].append(gene)
            except:
              motifs_all[m]=[gene]
            f.write(gene+"\t"+m+"\n")
        f.close()

        self.write_enrichment_table(threshold,out_path+"/pvalue_table_"+str(threshold*100)+".txt",motifs_all)

        net_pairs={}
        net_tfs={}
        all_pairs=sets.Set()
        all_tfs=sets.Set()
        all_genes=sets.Set()
        if targets == None:
          filter_targets=False
        else:
          filter_targets=True
          targets=[g.upper() for g in targets.genes]

        # using genes to motif mapping to get network in all conditions
        for net_name in self.networks.keys():
            net=self.networks[net_name]
            pairs=sets.Set()
            tfs=sets.Set()
            net_pairs[net_name]=pairs
            net_tfs[net_name]=tfs
            for tf in genes_motifs.keys():
                motifs=genes_motifs[tf]
                for m in motifs:
                    try:
                      for target in net[m]:
                        if (not filter_targets or (target in targets)):
                          pairs.add((tf,target))                          
                          tfs.add(tf)
                          all_genes.add(tf)
                          all_genes.add(target)
                    except Exception, e:
                        print("motif not in network: "+str(m)+" "+str(tf)+" "+str(e))
            all_pairs=all_pairs.union(pairs)
            all_tfs=all_tfs.union(tfs)
         
        #printing out network
        for net_name in net_pairs.keys():
            f=open(out_path+"/"+net_name+"_targets.txt","w")
            pairs_aux=net_pairs[net_name]
            for pair in all_pairs:
              #check if pair is active in the network  
              if pair in pairs_aux:
                f.write(pair[0]+"\t"+pair[1]+"\tactive\n")
              else:
                f.write(pair[0]+"\t"+pair[1]+"\tinactive\n")
            f.close()          
            f=open(out_path+"/"+net_name+"_genes.txt","w")
            for gene in all_genes:
              #check if gene is tf active in network
              if gene in net_tfs[net_name]:
                f.write(gene+"\ttf_active\n")
              elif gene in all_tfs:
                f.write(gene+"\ttf_inactive\n")
              else:
                f.write(gene+"\ttarget\n")
            f.close()       





# Python Libraries
from __future__ import division
from __future__ import print_function

import os
import sys
import time
import getpass
import argparse
import datetime
import matplotlib
matplotlib.use('Agg', warn=False)
import matplotlib.pyplot as plt

from boxplot import Boxplot
from lineplot import Lineplot
from jaccard_test import Jaccard
from projection_test import Projection
from intersection_test import Intersect
from shared_function import check_dir, print2, output_parameters,\
                            copy_em, list_all_index, output
from plotTools import Venn
dir = os.getcwd()
"""
Statistical analysis methods and plotting tools for ExperimentalMatrix

Author: Joseph C.C. Kuo
"""


def main():
    ###############################################################################
    ##### PARAMETERS ##############################################################
    ###############################################################################
    
    # Some general help descriptions
    ######### Some general plotting arguments descriptions ###############
    helpinput = 'The file name of the input Experimental Matrix file. Recommended to add more columns for more information for ploting. For example, cell type or factors. (default: %(default)s)'
    helpoutput = 'The directory name for the output files. For example, project name. (default: %(default)s)'
    helptitle = 'The title shown on the top of the plot and also the folder name. (default: %(default)s)'
    helpgroup = "Group the data by reads(needs 'factor' column), regions(needs 'factor' column), another name of column (for example, 'cell')in the header of experimental matrix, or None. (default: %(default)s)"
    helpgroupbb = "Group the data by any optional column (for example, 'cell') of experimental matrix, or None. (default: %(default)s)"
    helpsort = "Sort the data by reads(needs 'factor' column), regions(needs 'factor' column), another name of column (for example, 'cell')in the header of experimental matrix, or None. (default: %(default)s)"
    helpcolor = "Color the data by reads(needs 'factor' column), regions(needs 'factor' column), another name of column (for example, 'cell')in the header of experimental matrix, or None. (default: %(default)s)"
    helpcolorbb = "Color the data by any optional column (for example, 'cell') of experimental matrix, or None. (default: %(default)s)"
    helpDefinedColot = 'Define the specific colors with the given column "color" in experimental matrix. The color should be in the format of matplotlib.colors. For example, "r" for red, "b" for blue, or "(100, 35, 138)" for RGB. (default: %(default)s)'
    helpreference = 'The file name of the reference Experimental Matrix. Multiple references are acceptable. (default: %(default)s)'
    helpquery = 'The file name of the query Experimental Matrix. Multiple queries are acceptable. (default: %(default)s)'
    helpcol = "Group the data in columns by reads(needs 'factor' column), regions(needs 'factor' column), another name of column (for example, 'cell')in the header of experimental matrix, or None. (default: %(default)s)"
    helprow = "Group the data in rows by reads(needs 'factor' column), regions(needs 'factor' column), another name of column (for example, 'cell')in the header of experimental matrix, or None. (default: %(default)s)"
    helpmp = "Define the number of cores for parallel computation. (default: %(default)s)"
    parser = argparse.ArgumentParser(description='Provides various Statistical analysis methods and plotting tools for ExperimentalMatrix.\
    \nAuthor: Joseph C.C. Kuo, Ivan Gesteira Costa Filho', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    subparsers = parser.add_subparsers(help='sub-command help',dest='mode')
    
    ################### Projection test ##########################################
    parser_projection = subparsers.add_parser('projection',help='Projection test evaluates the association level by comparing to the random binomial model.')
    parser_projection.add_argument('-r', metavar='  ', help=helpreference)
    parser_projection.add_argument('-q', metavar='  ', help=helpquery)
    parser_projection.add_argument('-o', metavar='  ', help=helpoutput) 
    parser_projection.add_argument('-t', metavar='  ', default='projection_test', help=helptitle)
    parser_projection.add_argument('-g', metavar='  ', default=None, help=helpgroupbb )
    parser_projection.add_argument('-c', metavar='  ', default="regions", help=helpcolorbb )
    parser_projection.add_argument('-bg', metavar='  ', type=str, default=None, help="Define a BED file as background. If not defined, the background is whole genome according to the given organism. (default: %(default)s)")
    parser_projection.add_argument('-union', action="store_true", help='Take the union of references as background for binominal test. (default: %(default)s)')
    parser_projection.add_argument('-organism', metavar='  ', default='hg19', help='Define the organism. (default: %(default)s)')
    parser_projection.add_argument('-log', action="store_true", help='Set y axis of the plot in log scale. (default: %(default)s)')
    parser_projection.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_projection.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_projection.add_argument('-table', action="store_true", help='Store the tables of the figure in text format. (default: %(default)s)')
    parser_projection.add_argument('-bed', action="store_true", default=False, help='Output BED files for the regions of query which overlap the reference. (default: %(default)s)')
    parser_projection.add_argument('-pw', metavar='  ', type=int, default=5, help='Define the width of single panel. (default: %(default)s)')
    parser_projection.add_argument('-ph', metavar='  ', type=int, default=3, help='Define the height of single panel. (default: %(default)s)')
    parser_projection.add_argument('-cfp', metavar='  ', type=float, default=0.01, help='Define the cutoff of the proportion. (default: %(default)s)')
    
    ################### Intersect Test ##########################################
    parser_intersect = subparsers.add_parser('intersect',help='Intersection test provides various modes of intersection to test the association between references and queries.')
    parser_intersect.add_argument('-r', metavar='  ', help=helpreference)
    parser_intersect.add_argument('-q', metavar='  ', help=helpquery)
    parser_intersect.add_argument('-o', help=helpoutput)
    parser_intersect.add_argument('-t', metavar='  ', default='intersection_test', help=helptitle)
    parser_intersect.add_argument('-g', metavar='  ', default=None, help=helpgroupbb)
    parser_intersect.add_argument('-c', metavar='  ', default="regions", help=helpcolorbb)
    parser_intersect.add_argument('-organism', metavar='  ', default='hg19', help='Define the organism. (default: %(default)s)')
    parser_intersect.add_argument('-bg', metavar='  ', help="Define a BED file as background. If not defined, the background is whole genome according to the given organism. (default: %(default)s)")
    parser_intersect.add_argument('-m', metavar='  ', default="count", choices=['count','bp'],
                                  help="Define the mode of calculating intersection. 'count' outputs the number of overlapped regions.'bp' outputs the coverage(basepair) of intersection. (default: %(default)s)")
    parser_intersect.add_argument('-tc', metavar='  ', type=int, default=False, help="Define the threshold(in percentage) of reference length for intersection counting. For example, '20' means that the query which overlaps more than 20%% of reference is counted as intersection. (default: %(default)s)")
    parser_intersect.add_argument('-ex', metavar='  ', type=int, default=0, help="Define the extension(in bp) of reference length for intersection counting. For example, '20' means that each region of reference is extended by 20 bp in order to include proximal queries. (default: %(default)s)")
    parser_intersect.add_argument('-log', action="store_true", help='Set y axis of the plot in log scale.')
    parser_intersect.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_intersect.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_intersect.add_argument('-stest', metavar='  ', type=int, default= 0, help='Define the repetition time of random subregion test between reference and query. (default: %(default)s)')
    parser_intersect.add_argument('-mp', metavar='  ', default=4, type=int, help=helpmp)
    parser_intersect.add_argument('-pw', metavar='  ', type=int, default=3, help='Define the width of single panel. (default: %(default)s)')
    parser_intersect.add_argument('-ph', metavar='  ', type=int, default=3, help='Define the height of single panel. (default: %(default)s)')
    
    ################### Jaccard test ##########################################
    
    parser_jaccard = subparsers.add_parser('jaccard',help='Jaccard test evaluates the association level by comparing with jaccard index from repeating randomization.')
    
    parser_jaccard.add_argument('-o', help=helpoutput) 
    parser_jaccard.add_argument('-r', metavar='  ',help=helpreference)
    parser_jaccard.add_argument('-q', metavar='  ', help=helpquery)
    parser_jaccard.add_argument('-t', metavar='  ', default='jaccard_test', help=helptitle)
    parser_jaccard.add_argument('-rt',metavar='  ', type=int, default=500, help='Define how many times to run the randomization. (default: %(default)s)')
    parser_jaccard.add_argument('-g', default=None, help=helpgroupbb)
    parser_jaccard.add_argument('-c', default="regions", help=helpcolorbb)
    parser_jaccard.add_argument('-organism',default='hg19', help='Define the organism. (default: %(default)s)')
    parser_jaccard.add_argument('-nlog', action="store_false", help='Set y axis of the plot not in log scale. (default: %(default)s)')
    parser_jaccard.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_jaccard.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_jaccard.add_argument('-table', action="store_true", help='Store the tables of the figure in text format. (default: %(default)s)')
    parser_jaccard.add_argument('-pw', metavar='  ', type=int, default=3, help='Define the width of single panel. (default: %(default)s)')
    parser_jaccard.add_argument('-ph', metavar='  ', type=int, default=3, help='Define the height of single panel. (default: %(default)s)')
    
    ################### Combinatorial Test ##########################################
    parser_combinatorial = subparsers.add_parser('combinatorial',help='Combinatorial test compare all combinatorial possibilities from reference to test the association between references and queries.')
    
    parser_combinatorial.add_argument('-o', help=helpoutput)
    parser_combinatorial.add_argument('-r', metavar='  ', help=helpreference)
    parser_combinatorial.add_argument('-q', metavar='  ', help=helpquery)
    parser_combinatorial.add_argument('-t', metavar='  ', default='combinatorial_test', help=helptitle)
    parser_combinatorial.add_argument('-g', default=None, help=helpgroupbb)
    parser_combinatorial.add_argument('-c', default="regions", help=helpcolorbb)
    parser_combinatorial.add_argument('-organism',default='hg19', help='Define the organism. (default: %(default)s)')
    parser_combinatorial.add_argument('-bg', help="Define a BED file as background. If not defined, the background is whole genome according to the given organism. (default: %(default)s)")
    parser_combinatorial.add_argument('-m', default="count", choices=['count','bp'], help="Define the mode of calculating intersection. 'count' outputs the number of overlapped regions.'bp' outputs the coverage(basepair) of intersection. (default: %(default)s)")
    parser_combinatorial.add_argument('-tc', type=int, default=False, help="Define the threshold(in percentage) of reference length for intersection counting. For example, '20' means that the query which overlaps more than 20%% of reference is counted as intersection. (default: %(default)s)")
    parser_combinatorial.add_argument('-ex', type=int, default=0, help="Define the extension(in percentage) of reference length for intersection counting. For example, '20' means that each region of reference is extended by 20%% in order to include proximal queries. (default: %(default)s)")
    parser_combinatorial.add_argument('-log', action="store_true", help='Set y axis of the plot in log scale. (default: %(default)s)')
    parser_combinatorial.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_combinatorial.add_argument('-venn', action="store_true", help='Show the Venn diagram of the combinatorials of references. (default: %(default)s)')
    parser_combinatorial.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_combinatorial.add_argument('-stest', type=int, default= 0, help='Define the repetition time of random subregion test between reference and query. (default: %(default)s)')
    parser_combinatorial.add_argument('-pw', metavar='  ', type=int, default=3, help='Define the width of single panel. (default: %(default)s)')
    parser_combinatorial.add_argument('-ph', metavar='  ', type=int, default=3, help='Define the height of single panel. (default: %(default)s)')
    
    ################### Boxplot ##########################################
    
    parser_boxplot = subparsers.add_parser('boxplot',help='Boxplot based on the BAM and BED files for gene association analysis.')
    parser_boxplot.add_argument('input',help=helpinput)
    parser_boxplot.add_argument('-o', metavar='  ', help=helpoutput)
    parser_boxplot.add_argument('-t', metavar='  ', default='boxplot', help=helptitle)
    parser_boxplot.add_argument('-g', metavar='  ', default='reads', help=helpgroup)
    parser_boxplot.add_argument('-c', metavar='  ', default='regions', help=helpcolor)
    parser_boxplot.add_argument('-s', metavar='  ', default='None', help=helpsort)
    parser_boxplot.add_argument('-scol', action="store_true", help="Share y axis among columns. (default: %(default)s)")
    parser_boxplot.add_argument('-nlog', action="store_false", help='Set y axis of the plot not in log scale. (default: %(default)s)')
    parser_boxplot.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_boxplot.add_argument('-pw', metavar='  ', type=int, default=3, help='Define the width of single panel. (default: %(default)s)')
    parser_boxplot.add_argument('-ph', metavar='  ', type=int, default=3, help='Define the height of single panel. (default: %(default)s)')
    parser_boxplot.add_argument('-nqn', action="store_true", help='No quantile normalization in calculation. (default: %(default)s)')
    parser_boxplot.add_argument('-df', action="store_true", help="Show the difference of the two signals which share the same labels.The result is the subtraction of the first to the second. (default: %(default)s)")
    parser_boxplot.add_argument('-ylim', metavar='  ', type=int, default=None, help="Define the limit of y axis. (default: %(default)s)")
    parser_boxplot.add_argument('-p', metavar='  ', type=float, default=0.05, help='Define the significance level for multiple test.  (default: %(default)s)')
    parser_boxplot.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_boxplot.add_argument('-table', action="store_true", help='Store the tables of the figure in text format. (default: %(default)s)')
    
    ################### Lineplot ##########################################
    parser_lineplot = subparsers.add_parser('lineplot', help='Generate lineplot with various modes.')
    
    choice_center = ['midpoint','leftend','rightend','bothends','upstream','downstream'] 
    # Be consist as the arguments of GenomicRegionSet.relocate_regions
    
    parser_lineplot.add_argument('input', help=helpinput)
    parser_lineplot.add_argument('-o', help=helpoutput)
    parser_lineplot.add_argument('-ga', action="store_true", help="Use genetic annotation data as input regions (e.g. TSS, TTS, exons and introns) instead of the BED files in the input matrix.")
    parser_lineplot.add_argument('-t', metavar='  ', default='lineplot', help=helptitle)
    parser_lineplot.add_argument('-center', metavar='  ', choices=choice_center, default='midpoint', help='Define the center to calculate coverage on the regions. Options are: '+', '.join(choice_center) + '. (default: %(default)s) The bothend mode will flap the right end region for calculation.')
    parser_lineplot.add_argument('-row', metavar='  ', default='None', help=helprow)
    parser_lineplot.add_argument('-col', metavar='  ', default='regions', help=helpcol)
    parser_lineplot.add_argument('-c', metavar='  ', default='reads', help=helpcolor)
    parser_lineplot.add_argument('-e', metavar='  ', type=int, default=2000, help='Define the extend length of interested region for plotting. (default: %(default)s)')
    parser_lineplot.add_argument('-rs', metavar='  ', type=int, default=200, help='Define the readsize for calculating coverage. (default: %(default)s)')
    parser_lineplot.add_argument('-ss', metavar='  ', type=int, default=50, help='Define the stepsize for calculating coverage. (default: %(default)s)')
    parser_lineplot.add_argument('-bs', metavar='  ', type=int, default=100, help='Define the binsize for calculating coverage. (default: %(default)s)')
    parser_lineplot.add_argument('-log', action="store_true", help="Take log for the value before calculating average. (default: %(default)s)")
    parser_lineplot.add_argument('-scol', action="store_true", help="Share y axis among columns. (default: %(default)s)")
    parser_lineplot.add_argument('-srow', action="store_true", help="Share y axis among rows. (default: %(default)s)")
    parser_lineplot.add_argument('-organism', metavar='  ', default='hg19', help='Define the organism. (default: %(default)s)')
    parser_lineplot.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_lineplot.add_argument('-pw', metavar='  ', type=int, default=3, help='Define the width of single panel. (default: %(default)s)')
    parser_lineplot.add_argument('-ph', metavar='  ', type=int, default=3, help='Define the height of single panel. (default: %(default)s)')
    parser_lineplot.add_argument('-test', action="store_true", help="Sample only the first 10 regions in all BED files for testing. (default: %(default)s)")
    parser_lineplot.add_argument('-mp', metavar='  ', type=int, default=0, help="Perform multiprocessing for faster computation. (default: %(default)s)")
    parser_lineplot.add_argument('-df', action="store_true", help="Show the difference of the two signals which share the same labels.The result is the subtraction of the first to the second. (default: %(default)s)")
    parser_lineplot.add_argument('-dft', metavar='  ', default=None, help="Add one more tag for calculating difference. (default: %(default)s)")
    parser_lineplot.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_lineplot.add_argument('-table', action="store_true", help='Store the tables of the figure in text format. (default: %(default)s)')
    parser_lineplot.add_argument('-sense', action="store_true", help='Set the plot sense-specific. (default: %(default)s)')
    
    ################### Heatmap ##########################################
    parser_heatmap = subparsers.add_parser('heatmap', help='Generate heatmap with various modes.')
    
    choice_center = ['midpoint','leftend','rightend','bothends', 'upstream', 'downstream'] 
    # Be consist as the arguments of GenomicRegionSet.relocate_regions
    
    parser_heatmap.add_argument('input', help=helpinput)
    parser_heatmap.add_argument('-o', metavar='  ', help=helpoutput)
    parser_heatmap.add_argument('-ga', action="store_true", help="Use genetic annotation data as input regions (e.g. TSS, TTS, exons and introns) instead of the BED files in the input matrix. (default: %(default)s)")
    parser_heatmap.add_argument('-t', metavar='  ', default='heatmap', help=helptitle)
    parser_heatmap.add_argument('-center', metavar='  ', choices=choice_center, default='midpoint', 
                                 help='Define the center to calculate coverage on the regions. Options are: '+', '.join(choice_center) + 
                                 '.(Default:midpoint) The bothend mode will flap the right end region for calculation. (default: %(default)s)')
    parser_heatmap.add_argument('-sort', metavar='  ', type=int, default=None, help='Define the way to sort the signals.'+
                                'Default is no sorting at all, the signals arrange in the order of their position; '+
                                '"0" is sorting by the average ranking of all signals; '+
                                '"1" is sorting by the ranking of 1st column; "2" is 2nd and so on...  (default: %(default)s)')
    parser_heatmap.add_argument('-col', metavar='  ', default='regions', help=helpcol)
    parser_heatmap.add_argument('-c', metavar='  ', default='reads', help=helpcolor)
    parser_heatmap.add_argument('-row', metavar='  ', default='None', help=helprow)
    parser_heatmap.add_argument('-e', metavar='  ', type=int, default=2000, help='Define the extend length of interested region for plotting. (default: %(default)s)')
    parser_heatmap.add_argument('-rs', metavar='  ', type=int, default=200, help='Define the readsize for calculating coverage. (default: %(default)s)')
    parser_heatmap.add_argument('-ss', metavar='  ', type=int, default=50, help='Define the stepsize for calculating coverage. (default: %(default)s)')
    parser_heatmap.add_argument('-bs', metavar='  ', type=int, default=100, help='Define the binsize for calculating coverage. (default: %(default)s)')
    parser_heatmap.add_argument('-organism', metavar='  ', default='hg19', help='Define the organism. (default: %(default)s)')
    parser_heatmap.add_argument('-color', action="store_true", help=helpDefinedColot)
    parser_heatmap.add_argument('-log', action="store_true", help='Set colorbar in log scale. (default: %(default)s)')
    parser_heatmap.add_argument('-mp', action="store_true", help="Perform multiprocessing for faster computation. (default: %(default)s)")
    parser_heatmap.add_argument('-show', action="store_true", help='Show the figure in the screen. (default: %(default)s)')
    parser_heatmap.add_argument('-table', action="store_true", help='Store the tables of the figure in text format. (default: %(default)s)')
    
    ################### Venn Diagram ########################################
    parser_venn = subparsers.add_parser('venn', help='Generate Venn Diagram with peaks of gene list.')
    
    parser_venn.add_argument('-s1', metavar='  ', default=None, help="Define the file for gene set 1 (BED or gene list)")
    parser_venn.add_argument('-s2', metavar='  ', default=None, help="Define the file for gene set 2 (BED or gene list)")
    parser_venn.add_argument('-s3', metavar='  ', default=None, help="Define the file for gene set 3 (BED or gene list)")
    parser_venn.add_argument('-s4', metavar='  ', default=None, help="Define the file for gene set 3 (BED or gene list)")
    parser_venn.add_argument('-l1', metavar='  ', default=None, help="Define label on venn diagram for set 1")
    parser_venn.add_argument('-l2', metavar='  ', default=None, help="Define label on venn diagram for set 2")
    parser_venn.add_argument('-l3', metavar='  ', default=None, help="Define label on venn diagram for set 3")
    parser_venn.add_argument('-l4', metavar='  ', default=None, help="Define label on venn diagram for set 4")
    parser_venn.add_argument('-o', metavar='  ', help=helpoutput)
    parser_venn.add_argument('-t', metavar='  ', default='venn_diagram', help=helptitle)
    parser_venn.add_argument('-organism', metavar='  ', help='Define the organism. ')

    ################### Integration ##########################################
    parser_integration = subparsers.add_parser('integration', help='Provides some tools to deal with experimental matrix or other purposes.')
    parser_integration.add_argument('-ihtml', action="store_true", help='Integrate all the html files within the given directory and generate index.html for all plots.')
    parser_integration.add_argument('-l2m', help='Convert a given file list in txt format into a experimental matrix.')
    parser_integration.add_argument('-p', help='Define the folder of the output file.') 
    ################### Parsing the arguments ################################
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    elif len(sys.argv) == 2: 
        # retrieve subparsers from parser
        subparsers_actions = [action for action in parser._actions if isinstance(action, argparse._SubParsersAction)]
        # there will probably only be one subparser_action,but better save than sorry
        for subparsers_action in subparsers_actions:
            # get all subparsers and print help
            for choice, subparser in subparsers_action.choices.items():
                if choice == sys.argv[1]:
                    print("\nYou need more arguments.")
                    print("\nSubparser '{}'".format(choice))        
                    subparser.print_help()
        sys.exit(1)
    else:
        args = parser.parse_args()
        if not args.o:
            print("** Error: Please define the output directory (-o).")
            sys.exit(1)
        
        t0 = time.time()
        # Normalised output path
        args.o = os.path.normpath(os.path.join(dir,args.o))
        check_dir(args.o)
        check_dir(os.path.join(args.o, args.t))
        
        # Input parameters dictionary
        parameter = []
        parameter.append("Time: " + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
        parameter.append("User: " + getpass.getuser())
        parameter.append("\nCommand:\n\t$ " + " ".join(sys.argv))

        #################################################################################################
        ##### Main #####################################################################################
        #################################################################################################

        ################### Projection test ##########################################
        if args.mode == 'projection':
            # Fetching reference and query EM
            print2(parameter, "\n############# Projection Test #############")
            print2(parameter, "\tReference:        "+args.r)
            print2(parameter, "\tQuery:            "+args.q)
            print2(parameter, "\tOutput directory: "+os.path.basename(args.o))
            print2(parameter, "\tExperiment title: "+args.t)

            projection = Projection( args.r, args.q )
            projection.group_refque(args.g)
            projection.colors( args.c, args.color )
            
            if args.bg:
                print2(parameter, "\tBackground: "+args.bg)
                projection.set_background(bed_path=args.bg)
            if args.union: 
                projection.ref_union()
                projection.projection_test(organism=args.organism)
                print2(parameter, "\tTaking union of references as the background. ")
            else:
                projection.projection_test(organism=args.organism)
            
            # generate pdf
            projection.plot(args.log, args.pw, args.ph)
            output(f=projection.fig, directory = args.o, folder = args.t, filename="projection_test",
                   extra=plt.gci(),pdf=True,show=args.show)
            if args.bed:
                print2(parameter, "\tOutput BED files: "+"/".join(os.path.join(args.o, args.t, "bed").split("/")[-3:]))
                projection.output_interq(directory=os.path.join(args.o, args.t, "bed"))
            # generate html 
            projection.gen_html(args.o, args.t, args=args)
            
            if args.table:
                projection.table(directory = args.o, folder = args.t)
                
            print("\nAll related files are saved in:  "+ os.path.join(os.path.basename(args.o),args.t))
            t1 = time.time()
            print2(parameter,"\nTotal running time is : " + str(datetime.timedelta(seconds=round(t1-t0))))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.r, directory=args.o, folder=args.t, filename="reference_experimental_matrix.txt")
            copy_em(em=args.q, directory=args.o, folder=args.t, filename="query_experimental_matrix.txt")
            list_all_index(path=args.o)

        ###########################################################################
        ################### Intersect Test ##########################################
        if args.mode == 'intersect':
            print2(parameter, "\n############ Intersection Test ############")
            print2(parameter, "\tReference:        "+args.r)
            print2(parameter, "\tQuery:            "+args.q)
            print2(parameter, "\tOutput directory: "+os.path.basename(args.o))
            print2(parameter, "\tExperiment title: "+args.t)
            
            
            # Fetching reference and query EM
            inter = Intersect(args.r,args.q, mode_count=args.m, organism=args.organism)
            
            # Grouping
            inter.group_refque(args.g)
            # Setting background
            inter.background(args.bg)

            # Extension
            if args.ex == 0: pass
            elif args.ex > 0: inter.extend_ref(args.ex)
            elif args.ex < 0: 
                print("\n**** extension percentage(-ex) should be positive value, not negative.\n")
                sys.exit(1)
            
            inter.colors(args.c, args.color)
            print("\tProcessing data.", end="")
            sys.stdout.flush()
            inter.count_intersect(threshold=args.tc)
            
            # generate pdf
            print("\n\tGenerate graphics...")
            inter.barplot(logt=args.log)
            output(f=inter.bar, directory = args.o, folder = args.t, filename="intersection_bar",
                   extra=plt.gci(), pdf=True,show=args.show)
            inter.stackedbar()
            output(f=inter.sbar, directory = args.o, folder = args.t, filename="intersection_stackedbar",
                   extra=plt.gci(), pdf=True,show=args.show)
            inter.barplot(logt=args.log, percentage=True)
            output(f=inter.bar, directory = args.o, folder = args.t, filename="intersection_barp",
                   extra=plt.gci(), pdf=True,show=args.show)
            
            if args.stest > 0:
                print("\tStatistical testing by randomizing the regions...")
                inter.stest(repeat=args.stest,threshold=args.tc, mp=args.mp)
            
            # generate html
            inter.gen_html(directory=args.o, title=args.t, align=50, args=args)
            
            t1 = time.time()
            print2(parameter, "\nAll related files are saved in:  "+ os.path.join(os.path.basename(args.o),args.t))
            print2(parameter,"\nTotal running time is : " + str(datetime.timedelta(seconds=round(t1-t0))))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.r, directory=args.o, folder=args.t, filename="reference_experimental_matrix.txt")
            copy_em(em=args.q, directory=args.o, folder=args.t, filename="query_experimental_matrix.txt")
            list_all_index(path=args.o)

        ###########################################################################
        ################### Jaccard test ##########################################
        if args.mode == "jaccard":
            """Return the jaccard test of every possible comparisons between two ExperimentalMatrix. 
            
            Method:
            The distribution of random jaccard index is calculated by randomizing query for given times. 
            Then, we compare the real jaccard index to the distribution and formulate p-value as 
            p-value = (# random jaccard > real jaccard)/(# random jaccard)
            
            """
            print("\n############## Jaccard Test ###############")
            jaccard = Jaccard(args.r,args.q)
            jaccard.group_refque(args.g)
            jaccard.colors(args.c, args.color)
            
            # jaccard test
            jaccard.jaccard_test(args.rt, args.organism)
            parameter = parameter + jaccard.parameter
            t1 = time.time()
            # ploting and generate pdf
            jaccard.plot(logT=args.nlog)
            for i,f in enumerate(jaccard.fig):
                output(f=f, directory = args.o, folder = args.t, filename="jaccard_test"+str(i+1),
                       extra=plt.gci(),pdf=True,show=args.show)
            # generate html
            jaccard.gen_html(args.o, args.t)
            
            if args.table:
                jaccard.table(directory = args.o, folder = args.t)
            
            print("\nAll related files are saved in:  "+ os.path.join(dir,args.o,args.t))
            print2(parameter,"\nTotal running time is : " + str(datetime.timedelta(seconds=round(t1-t0))))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.r, directory=args.o, folder=args.t, filename="Reference_experimental_matrix.txt")
            copy_em(em=args.q, directory=args.o, folder=args.t, filename="Query_experimental_matrix.txt")
            list_all_index(path=args.o)

        ###########################################################################
        ################### Combinatorial Test ##########################################
        if args.mode == 'combinatorial':
            print("\n############ Combinatorial Test ############")
            # Fetching reference and query EM
            #comb = Combinatorial(args.r,args.q, mode_count=args.m, organism=args.organism)
            inter = Intersect(args.r,args.q, mode_count=args.m, organism=args.organism)
            # Setting background
            inter.background(args.bg)
            # Grouping
            inter.group_refque(args.g)
            
            # Extension
            if args.ex == 0: pass
            elif args.ex > 0: inter.extend_ref(args.ex)
            elif args.ex < 0: 
                print("\n**** extension percentage(-ex) should be positive value, not negative.\n")
                sys.exit(1)
            # Combinatorial 
            print2(parameter, "Generating all combinatorial regions for further analysis...")
            inter.combinatorial()
            inter.count_intersect(threshold=args.tc, frequency=True)
            
            # generate pdf
            inter.colors_comb()
            #inter.barplot(args.log)
            #output(f=inter.bar, directory = args.output, folder = args.title, filename="intersection_bar",extra=plt.gci(),pdf=True,show=args.show)
            #if args.stackedbar:
            #inter.colors(args.c, args.color,ref_que = "ref")
            inter.comb_stacked_plot()
            

            output(f=inter.sbar, directory = args.o, folder = args.t, filename="intersection_stackedbar",
                   extra=plt.gci(), pdf=True, show=args.show)
            if args.venn:
                inter.comb_venn(directory = os.path.join(args.o, args.t))
                
            #if args.lineplot:
            #    inter.comb_lineplot()
            if args.stest > 0:
                inter.stest(repeat=args.stest,threshold=args.tc)
            # generate html
            inter.gen_html_comb(directory=args.o, title=args.t, align=50, args=args)
            
            #parameter = parameter + inter.parameter
            t1 = time.time()
            print("\nAll related files are saved in:  "+ os.path.join(dir,args.o,args.t))
            print2(parameter,"\nTotal running time is : " + str(datetime.timedelta(seconds=round(t1-t0))))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.r, directory=args.o, folder=args.t, filename="Reference_experimental_matrix.txt")
            copy_em(em=args.q, directory=args.o, folder=args.t, filename="Query_experimental_matrix.txt")
            #list_all_index(path=args.o)

        ###########################################################################
        ################### Boxplot ##########################################
        if args.mode == 'boxplot':
            print("\n################# Boxplot #################")
            boxplot = Boxplot(args.input, fields=[args.g, args.s, args.c], title=args.t, df=args.df)
            
            print2(parameter,"\nStep 1/5: Combining all regions")
            boxplot.combine_allregions()
            print2(parameter,"    " + str(len(boxplot.all_bed)) + " regions from all bed files are combined.")
            t1 = time.time()
            print2(parameter,"    --- finished in {0} secs\n".format(round(t1-t0)))
            
            # Coverage of reads on all_bed
            print2(parameter,"Step 2/5: Calculating coverage of each bam file on all regions")
            boxplot.bedCoverage() 
            t2 = time.time()
            print2(parameter,"    --- finished in {0} (H:M:S)\n".format(datetime.timedelta(seconds=round(t2-t1))))
            
            # Quantile normalization
            print2(parameter,"Step 3/5: Quantile normalization of all coverage table")
            if args.nqn:
                print2(parameter,"    No quantile normalization.")
                boxplot.norm_table = boxplot.all_table
            else: boxplot.quantile_normalization()
            t3 = time.time()
            print2(parameter,"    --- finished in {0} secs\n".format(round(t3-t2)))
            
            # Generate individual table for each bed
            print2(parameter,"Step 4/5: Constructing different tables for box plot")
            boxplot.tables_for_plot()
            # if args.table: boxplot.print_plot_table(directory = args.o, folder = args.t)
            t4 = time.time()
            print2(parameter,"    --- finished in {0} secs\n".format(round(t4-t3)))
            
            # Plotting
            print2(parameter,"Step 5/5: Plotting")
            boxplot.group_tags(groupby=args.g, sortby=args.s, colorby=args.c)
            
            boxplot.group_data(directory = args.o, folder = args.t, log=args.nlog)
            boxplot.color_map(colorby=args.c, definedinEM=args.color)
            boxplot.plot(title=args.t, logT=args.nlog, scol=args.scol, ylim=args.ylim, pw=args.pw, ph=args.ph)
            if args.table: boxplot.print_table(directory=args.o, folder=args.t)
            output(f=boxplot.fig, directory = args.o, folder = args.t, filename="boxplot",extra=plt.gci(),pdf=True,show=args.show)
            # HTML
            boxplot.gen_html(args.o, args.t, align = 50)
            t5 = time.time()
            print2(parameter,"    --- finished in {0} secs\n".format(round(t5-t4)))
            print2(parameter,"Total running time is: " + str(datetime.timedelta(seconds=round(t5-t0))) + " (H:M:S)\n")
            print("\nAll related files are saved in:  "+ os.path.join(dir,args.o,args.t))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.input, directory=args.o, folder=args.t)
            list_all_index(path=args.o)

        ################### Lineplot #########################################
        if args.mode == 'lineplot':
            if args.scol and args.srow:
                print("** Err: -scol and -srow cannot be used simutaneously.")
                sys.exit(1)

            print("\n################ Lineplot #################")
            # Read experimental matrix
            t0 = time.time()
            # if "reads" not in (args.col, args.c, args.row):
            #     print("Please add 'reads' tag as one of grouping, sorting, or coloring argument.")
            #     sys.exit(1)
            # if "regions" not in (args.col, args.c, args.row):
            #     print("Please add 'regions' tag as one of grouping, sorting, or coloring argument.")
            #     sys.exit(1)

            if not os.path.isfile(args.input):
                print("Please check the input experimental matrix again. The given path is wrong.")
                sys.exit(1)
            print2(parameter, "Parameters:\tExtend length:\t"+str(args.e))
            print2(parameter, "\t\tRead size:\t"+str(args.rs))
            print2(parameter, "\t\tBin size:\t"+str(args.bs))
            print2(parameter, "\t\tStep size:\t"+str(args.ss))
            print2(parameter, "\t\tCenter mode:\t"+str(args.center+"\n"))
            
            lineplot = Lineplot(EMpath=args.input, title=args.t, annotation=args.ga, 
                                organism=args.organism, center=args.center, extend=args.e, rs=args.rs, 
                                bs=args.bs, ss=args.ss, df=args.df, dft=args.dft, fields=[args.col,args.row,args.c],
                                test=args.test, sense=args.sense)
            # Processing the regions by given parameters
            print2(parameter, "Step 1/3: Processing regions by given parameters")
            lineplot.relocate_bed()
            t1 = time.time()
            print2(parameter, "\t--- finished in {0} secs".format(str(round(t1-t0))))
            
            if args.mp > 0: print2(parameter, "\nStep 2/3: Calculating the coverage to all reads and averaging with multiprocessing ")
            else: print2(parameter, "\nStep 2/3: Calculating the coverage to all reads and averaging")
            lineplot.group_tags(groupby=args.col, sortby=args.row, colorby=args.c)
            lineplot.gen_cues()
            lineplot.coverage(sortby=args.row, mp=args.mp, log=args.log)
            t2 = time.time()
            print2(parameter, "\t--- finished in {0} (H:M:S)".format(str(datetime.timedelta(seconds=round(t2-t1)))))
            
            # Plotting
            print2(parameter, "\nStep 3/3: Plotting the lineplots")
            lineplot.colormap(colorby = args.c, definedinEM = args.color)
            lineplot.plot(groupby=args.col, colorby=args.c, output=args.o, printtable=args.table, 
                          scol=args.scol, srow=args.srow, w=args.pw, h=args.ph)
            output(f=lineplot.fig, directory = args.o, folder = args.t, filename="lineplot",extra=plt.gci(),pdf=True,show=args.show)
            lineplot.gen_html(args.o, args.t)
            t3 = time.time()
            print2(parameter, "\t--- finished in {0} secs".format(str(round(t3-t2))))
            print2(parameter, "\nTotal running time is : " + str(datetime.timedelta(seconds=round(t3-t0))) + "(H:M:S)\n")
            print("\nAll related files are saved in:  "+ os.path.join(dir,args.o,args.t))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.input, directory=args.o, folder=args.t)
            list_all_index(path=args.o)

        ################### Heatmap ##########################################
        if args.mode=='heatmap':
            print("\n################# Heatmap #################")
            # Most part of heat map are the same as lineplot, so it share the same class as lineplot
            # Read experimental matrix
            t0 = time.time()

            if "reads" not in (args.col, args.c, args.row):
                print("Please add 'reads' tag as one of grouping, sorting, or coloring argument.")
                sys.exit(1)
            if "regions" not in (args.col, args.c, args.row):
                print("Please add 'regions' tag as one of grouping, sorting, or coloring argument.")
                sys.exit(1)
            print2(parameter, "Parameters:\tExtend length:\t"+str(args.e))
            print2(parameter, "\t\tRead size:\t"+str(args.rs))
            print2(parameter, "\t\tBin size:\t"+str(args.bs))
            print2(parameter, "\t\tStep size:\t"+str(args.ss))
            print2(parameter, "\t\tCenter mode:\t"+str(args.center+"\n"))

            lineplot = Lineplot(EMpath=args.input, title=args.t, annotation=args.ga, 
                                organism=args.organism, center=args.center, extend=args.e, rs=args.rs, 
                                bs=args.bs, ss=args.ss, df=False, fields=[args.col,args.row,args.c])
            # Processing the regions by given parameters
            print2(parameter, "Step 1/4: Processing regions by given parameters")
            lineplot.relocate_bed()
            t1 = time.time()
            print2(parameter, "    --- finished in {0} secs".format(str(round(t1-t0))))
            
            if args.mp: print2(parameter, "\nStep 2/4: Calculating the coverage to all reads and averaging with multiprocessing ")
            else: print2(parameter, "\nStep 2/4: Calculating the coverage to all reads and averaging")
            lineplot.group_tags(groupby=args.col, sortby=args.row, colorby=args.c)
            lineplot.gen_cues()
            lineplot.coverage(sortby=args.s, heatmap=True, logt=args.log, mp=args.mp)
            t2 = time.time()
            print2(parameter, "    --- finished in {0} (h:m:s)".format(str(datetime.timedelta(seconds=round(t2-t1)))))
            
            # Sorting 
            print2(parameter, "\nStep 3/4: Sorting the data for heatmap")
            lineplot.hmsort(sort=args.sort)
            t3 = time.time()
            print2(parameter, "    --- finished in {0} (h:m:s)".format(str(datetime.timedelta(seconds=round(t3-t2)))))
            
            # Plotting
            print2(parameter, "\nStep 4/4: Plotting the heatmap")
            lineplot.hmcmlist(colorby = args.c, definedinEM = args.color)
            lineplot.heatmap(args.log)
            for i, name in enumerate(lineplot.hmfiles):
                output(f=lineplot.figs[i], directory = args.o, folder = args.t, filename=name,pdf=True,show=args.show)
            lineplot.gen_htmlhm(args.o, args.t)
            t4 = time.time()
            print2(parameter, "    --- finished in {0} secs".format(str(round(t4-t3))))
            print2(parameter, "\nTotal running time is : " + str(datetime.timedelta(seconds=round(t4-t0))) + "(H:M:S)\n")
            print("\nAll related files are saved in:  "+ os.path.join(dir,args.o,args.t))
            output_parameters(parameter, directory = args.o, folder = args.t, filename="parameters.txt")
            copy_em(em=args.input, directory=args.o, folder=args.t)
            list_all_index(path=args.o)

        
        ################### Venn Diagram ##########################################
        if args.mode=='venn':
            print("\n################# Venn Diagram ###############")
            if not os.path.exists(os.path.join(args.o, args.t)):
                os.makedirs(os.path.join(args.o, args.t))
            sets = [s for s in [args.s1, args.s2, args.s3, args.s4] if s ]
            venn = Venn(sets=sets, organism=args.organism)
            f = venn.venn_diagram(directory=args.o, title=args.t,labels = [args.l1, args.l2, args.l3, args.l4])
            output(f=f, directory = args.o, folder = args.t, filename="venn", pdf=True)

        ################### Integration ##########################################
        if args.mode=='integration':
            print("\n################# Integration ###############")
            if args.ihtml:
                list_all_index(path=args.o)



###################################################################################################
# Libraries
###################################################################################################

# Python
from __future__ import print_function
import numpy as np
import math
from sklearn import metrics
from scipy.integrate import trapz
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import pylab

from pysam import Samfile

# Internal
from rgt.GenomicRegionSet import GenomicRegionSet
from rgt.Util import OverlapType
from ..Util import GenomeData

"""
Evaluate the footprints prediction using TF ChIP-seq or expression data.

Authors: Eduardo G. Gusmao, Zhijian Li
"""


class Evaluation:
    """
    Contains two different methodologies: TF ChIP-seq Based Evaluation

    """

    def __init__(self, tf_name, tfbs_file, footprint_file, footprint_name, footprint_type,
                 print_roc_curve, print_pr_curve, output_location, alignment_file, organism):
        self.tf_name = tf_name
        self.tfbs_file = tfbs_file
        self.footprint_file = footprint_file.split(",")
        self.footprint_name = footprint_name.split(",")
        self.footprint_type = footprint_type.split(",")
        self.print_roc_curve = print_roc_curve
        self.print_pr_curve = print_pr_curve
        self.output_location = output_location
        self.alignment_file = alignment_file
        self.organism = organism
        if self.output_location[-1] != "/":
            self.output_location += "/"

    def chip_evaluate(self):
        """
        This evaluation methodology uses motif-predicted binding sites (MPBSs) together with TF ChIP-seq data
        to evaluate the footprint predictions.

        return:
        """

        # Evaluate Statistics
        fpr = dict()
        tpr = dict()
        roc_auc = dict()
        roc_auc_1 = dict()
        roc_auc_2 = dict()
        recall = dict()
        precision = dict()
        prc_auc = dict()

        if "SEG" in self.footprint_type:
            mpbs_regions = GenomicRegionSet("TFBS")
            mpbs_regions.read_bed(self.tfbs_file)
            mpbs_regions.sort()

            # Verifying the maximum score of the MPBS file
            max_score = -99999999
            for region in iter(mpbs_regions):
                score = int(region.data)
                if score > max_score:
                    max_score = score
            max_score += 1

        for i in range(len(self.footprint_file)):
            footprints_regions = GenomicRegionSet("Footprints Prediction")
            footprints_regions.read_bed(self.footprint_file[i])

            # Sort footprint prediction bed files
            footprints_regions.sort()

            if self.footprint_type[i] == "SEG":
                # Increasing the score of MPBS entry once if any overlaps found in the predicted footprints.
                increased_score_mpbs_regions = GenomicRegionSet("Increased Regions")
                intersect_regions = mpbs_regions.intersect(footprints_regions, mode=OverlapType.ORIGINAL)
                for region in iter(intersect_regions):
                    region.data = str(int(region.data) + max_score)
                    increased_score_mpbs_regions.add(region)


                # Keep the score of remained MPBS entry unchanged
                without_intersect_regions = mpbs_regions.subtract(footprints_regions, whole_region=True)
                for region in iter(without_intersect_regions):
                    increased_score_mpbs_regions.add(region)

                increased_score_mpbs_regions.sort_score()

                fpr[i], tpr[i], roc_auc[i], roc_auc_1[i], roc_auc_2[i] = self.roc_curve(increased_score_mpbs_regions)
                recall[i], precision[i], prc_auc[i] = self.precision_recall_curve(increased_score_mpbs_regions)
            elif self.footprint_type[i] == "SC":
                footprints_regions.sort_score()
                fpr[i], tpr[i], roc_auc[i], roc_auc_1[i], roc_auc_2[i] = self.roc_curve(footprints_regions)
                recall[i], precision[i], prc_auc[i] = self.precision_recall_curve(footprints_regions)

        # Output the statistics results into text
        stats_fname = self.output_location + self.tf_name + "_stats.txt"
        stats_header = ["METHOD", "AUC_100", "AUC_10", "AUC_1", "AUPR"]
        with open(stats_fname, "w") as stats_file:
            stats_file.write("\t".join(stats_header) + "\n")
            for i in range(len(self.footprint_name)):
                stats_file.write(self.footprint_name[i] + "\t" + str(roc_auc[i]) + "\t" + str(roc_auc_1[i]) + "\t"
                                 + str(roc_auc_2[i]) + "\t" + str(prc_auc[i]) + "\n")

        # Output the curves
        if self.print_roc_curve:
            label_x = "False Positive Rate"
            label_y = "True Positive Rate"
            curve_name = "ROC"
            self.plot_curve(fpr, tpr, roc_auc, label_x, label_y, self.tf_name, curve_name)
        if self.print_pr_curve:
            label_x = "Recall"
            label_y = "Precision"
            curve_name = "PRC"
            self.plot_curve(recall, precision, prc_auc, label_x, label_y, self.tf_name, curve_name)

        self.output_points(self.tf_name, fpr, tpr, recall, precision)

    def plot_curve(self, data_x, data_y, stats, label_x, label_y, tf_name, curve_name):
        color_list = ["#000000", "#000099", "#006600", "#990000", "#660099", "#CC00CC", "#222222", "#CC9900",
                      "#FF6600", "#0000CC", "#336633", "#CC0000", "#6600CC", "#FF00FF", "#555555", "#CCCC00",
                      "#FF9900", "#0000FF", "#33CC33", "#FF0000", "#663399", "#FF33FF", "#888888", "#FFCC00",
                      "#663300", "#009999", "#66CC66", "#FF3333", "#9933FF", "#FF66FF", "#AAAAAA", "#FFCC33",
                      "#993300", "#00FFFF", "#99FF33", "#FF6666", "#CC99FF", "#FF99FF", "#CCCCCC", "#FFFF00"]

        matplotlib.use('Agg')
        # Creating figure
        fig = plt.figure(figsize=(8, 5), facecolor='w', edgecolor='k')
        ax = fig.add_subplot(111)
        for i in range(len(self.footprint_name)):
            ax.plot(data_x[i], data_y[i], color=color_list[i],
                    label=self.footprint_name[i] + ": " + str(round(stats[i], 4)))

        # Plot red diagonal line
        ax.plot([0, 1.0], [0, 1.0], color="#CCCCCC", linestyle="--", alpha=1.0)

        # Line legend
        leg = ax.legend(bbox_to_anchor=(1.0, 0.0, 1.0, 1.0), loc=2, ncol=2, borderaxespad=0., title="AUC",
                        prop={'size': 4})
        for e in leg.legendHandles:
            e.set_linewidth(2.0)

        # Titles and Axis Labels
        ax.set_title(tf_name)
        ax.set_xlabel(label_x)
        ax.set_ylabel(label_y)

        # Ticks
        ax.grid(True, which='both')
        ax.set_xticks(np.arange(0.0, 1.01, 0.1))
        ax.set_yticks(np.arange(0.0, 1.01, 0.1))
        for tick in ax.xaxis.get_major_ticks():
            tick.label.set_fontsize(10)
        for tick in ax.yaxis.get_major_ticks():
            tick.label.set_fontsize(10)

        # Axis limits
        pylab.xlim([0, 1.0])
        pylab.ylim([0, 1.0])

        # Saving figure
        figure_name = self.output_location + tf_name + "_" + curve_name + ".png"
        fig.savefig(figure_name, format="png", dpi=300, bbox_inches='tight', bbox_extra_artists=[leg])

    def roc_curve(self, sort_score_regions):
        count_x = 0
        count_y = 0
        fpr = [count_x]
        tpr = [count_y]
        for region in iter(sort_score_regions):
            if str(region.name).split(":")[-1] == "Y":
                count_y += 1
                fpr.append(count_x)
                tpr.append(count_y)
            else:
                count_x += 1
                fpr.append(count_x)
                tpr.append(count_y)
        fpr = [e * (1.0 / count_x) for e in fpr]
        tpr = [e * (1.0 / count_y) for e in tpr]

        # Evaluating 100% AUC
        roc_auc = metrics.auc(fpr, tpr)

        # Evaluating 10% AUC
        fpr_auc = 0.1
        fpr_1 = list()
        tpr_1 = list()
        for idx in range(0, len(fpr)):
            if (fpr[idx] > fpr_auc): break
            fpr_1.append(fpr[idx])
            tpr_1.append(tpr[idx])
        roc_auc_1 = metrics.auc(self.standardize(fpr_1), tpr_1)

        # Evaluating 1% AUC
        fpr_auc = 0.01
        fpr_2 = list()
        tpr_2 = list()
        for idx in range(0, len(fpr)):
            if (fpr[idx] > fpr_auc):
                break
            fpr_2.append(fpr[idx])
            tpr_2.append(tpr[idx])
        roc_auc_2 = metrics.auc(self.standardize(fpr_2), tpr_2)

        return fpr, tpr, roc_auc, roc_auc_1, roc_auc_2

    def precision_recall_curve(self, sort_score_regions):
        count_x = 0
        count_y = 0
        precision = [0.0]
        recall = [0.0]
        for region in iter(sort_score_regions):
            if str(region.name).split(":")[-1] == "Y":
                count_y += 1
                precision.append(float(count_y) / (count_x + count_y))
                recall.append(count_y)
            else:
                count_x += 1
                precision.append(float(count_y) / (count_x + count_y))
                recall.append(count_y)

        recall = [e * (1.0 / count_y) for e in recall]
        precision.append(0.0)
        recall.append(1.0)
        auc = (abs(trapz(recall, precision)))

        return recall, precision, auc

    def standardize(self, vector):
        maxN = max(vector)
        minN = min(vector)
        return [(e - minN) / (maxN - minN) for e in vector]

    def optimize_roc_points(self, fpr, tpr, max_points=1000):
        new_fpr = dict()
        new_tpr = dict()
        for i in range(len(self.footprint_name)):
            if (len(fpr[i]) > max_points):
                new_idx_list = [int(math.ceil(e)) for e in np.linspace(0, len(fpr[i]) - 1, max_points)]
                new_idx_fpr = []
                new_idx_tpr = []
                for j in new_idx_list:
                    new_idx_fpr.append(fpr[i][j])
                    new_idx_tpr.append(tpr[i][j])
                new_fpr[i] = new_idx_fpr
                new_tpr[i] = new_idx_tpr
            else:
                new_fpr[i] = fpr[i]
                new_tpr[i] = tpr[i]

        return new_fpr, new_tpr

    def optimize_pr_points(self, recall, precision, max_points=1000):
        new_recall = dict()
        new_precision = dict()

        for i in range(len(self.footprint_name)):
            data_recall = []
            data_precision = []
            for j in range(min(max_points, len(recall[i]))):
                data_recall.append(recall[i].pop(0))
                data_precision.append(precision[i].pop(0))
            if len(recall[i]) > max_points:
                new_idx_list = [int(math.ceil(e)) for e in np.linspace(0, len(recall[i]) - 1, max_points)]
                for j in new_idx_list:
                    data_recall.append(recall[i][j])
                    data_precision.append(precision[i][j])
                new_recall[i] = data_recall
                new_precision[i] = data_precision
            else:
                new_recall[i] = data_recall + recall[i]
                new_precision[i] = data_precision + precision[i]

        return new_recall, new_precision

    def output_points(self, tf_name, fpr, tpr, recall, precision):
        roc_fname = self.output_location + tf_name + "_roc.txt"
        new_fpr, new_tpr = self.optimize_roc_points(fpr, tpr)
        header = list()
        len_vec = list()
        for i in range(len(self.footprint_name)):
            header.append(self.footprint_name[i] + "_FPR")
            header.append(self.footprint_name[i] + "_TPR")
            len_vec.append(len(new_fpr[i]))

        with open(roc_fname, "w") as roc_file:
            roc_file.write("\t".join(header) + "\n")
            max_idx = len_vec.index(max(len_vec))
            for j in range(len(new_fpr[max_idx])):
                to_write = list()
                for i in range(len(self.footprint_name)):
                    if j >= len(new_fpr[i]):
                        to_write.append("NA")
                    else:
                        to_write.append(str(new_fpr[i][j]))
                    if j >= len(new_tpr[i]):
                        to_write.append("NA")
                    else:
                        to_write.append(str(new_tpr[i][j]))
                roc_file.write("\t".join(to_write) + "\n")

        prc_fname = self.output_location + tf_name + "_prc.txt"
        new_recall, new_precision = self.optimize_pr_points(recall, precision)
        header = list()
        len_vec = list()
        for i in range(len(self.footprint_name)):
            header.append(self.footprint_name[i] + "_REC")
            header.append(self.footprint_name[i] + "_PRE")
            len_vec.append(len(new_recall[i]))

        with open(prc_fname, "w") as prc_file:
            prc_file.write("\t".join(header) + "\n")
            max_idx = len_vec.index(max(len_vec))
            for j in range(len(new_recall[max_idx])):
                to_write = list()
                for i in range(len(self.footprint_name)):
                    if j >= len(new_recall[i]):
                        to_write.append("NA")
                    else:
                        to_write.append(str(new_recall[i][j]))
                    if j >= len(new_precision[i]):
                        to_write.append("NA")
                    else:
                        to_write.append(str(new_precision[i][j]))
                prc_file.write("\t".join(to_write) + "\n")



#!/usr/bin/env python
#  -*- coding: utf-8 -*-
"""
Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Consider the 2x1 contingency table with x and y 
and fixed marginal column sum n=x+y. Value x follows h1, 
value y follows h2.


@author: Manuel Allhoff
"""

from __future__ import print_function
from scipy.stats import binom
from math import log
from sklearn.utils.extmath import logsumexp
import numpy as np
from rgt.THOR.neg_bin import NegBin

lookup_pmf = {}
lookup_pvalue = {}


def get_value(x, distr):
    if distr['distr_name'] == 'binomial':
        if lookup_pmf.has_key(x):
            return lookup_pmf[x]
        else:
            v = binom.pmf(x, distr['n'], distr['p'])
            lookup_pmf[x] = v
            return v


def get_log_value(x, distr):
    if distr['distr_name'] == 'binomial':
        if lookup_pmf.has_key(x):
            return lookup_pmf[x]
        else:
            v = binom.logpmf(x, distr['n'], distr['p'])
            lookup_pmf[x] = v
            return v
    if distr['distr_name'] == 'nb':
        return distr['distr'].logpdf(x)
        # return nbinom.logpmf(x, distr['n'], distr['p'])


def _comp(i, x, side, current_p, p):
    if side == 'r':
        return i <= x
    elif side == 'l':
        return i >= x
    else:
        return current_p >= p


def compute_pvalue(distr, N, side, current_p, x):
    """Compute log2 pvalue"""
    sum_num = []
    sum_denum = []
    it = range(N / 2 + 1) if side == 'r' else range(N + 1, -1, -1)

    for i in it:
        p1 = get_log_value(i, distr)
        p2 = get_log_value(N - i, distr)
        p = p1 + p2

        if _comp(i, x, side, current_p, p):
            # if p > current_p:
            sum_num.append(p)
        sum_denum.append(p)

    if distr['distr_name'] == 'nb':
        sum_num = map(lambda x: float(x), sum_num)
        sum_denum = map(lambda x: float(x), sum_denum)

    return logsumexp(np.array(sum_num)) - (log(2) + logsumexp(np.array(sum_denum)))


def get_log_pvalue_new(x, y, side, distr):
    """compute log10 p-value"""
    N = x + y
    if side == 'l':
        x, y = y, x
        side = 'r'

    if lookup_pvalue.has_key((x, y, 'r')):
        return lookup_pvalue[(x, y, 'r')]
    else:
        current_p = get_log_value(x, distr) + get_log_value(y, distr)
        pvalue = compute_pvalue(distr, N, side, current_p, x) / log(10)
        lookup_pvalue[(x, y, side)] = pvalue
        return pvalue


def change_nb_WP2NB1(n, p):
    alpha = 1. / n
    mu = (1. / p - 1) / alpha

    return mu, alpha


def change_nb_NB12WP(mu, alpha):
    alpha = float(alpha)
    mu = float(mu)
    p = 1. / (1 + mu * alpha)
    n = 1. / alpha

    return n, p


if __name__ == '__main__':
    mu = 1.03882161264
    alpha = 0.1

    m = NegBin(mu, alpha)
    distr = {'distr_name': 'nb', 'distr': m}

    # ,0.510793370086
    for x, y in [([800, 900], [600, 500]), ([200, 190], [40, 50])]:
        side = 'l' if x > y else 'r'
        var = np.var(x + y)
        mu = np.mean(x + y)
        alpha = max((var - mu) / np.square(mu), 0.00000000001)
        m = NegBin(mu, alpha)
        distr = {'distr_name': 'nb', 'distr': m}
        print(x, y, -get_log_pvalue_new(int(sum(x)), int(sum(y)), side, distr), sep='\t')


        # n = 90
        # p = 0.01
        # distr={'distr_name': 'binomial', 'p':p, 'n':n}

        # n, p = change_nb_NB12WP(mu, alpha)

        # n, p = 10, 0.1 #working
        # print(n, p)
        # distr={'distr_name': 'nb', 'p': p, 'n': n}

        # for i in range(10):
        # print(nbinom.logpmf(i, distr['n'], distr['p']), m.logpdf(i), fabs(nbinom.logpmf(i, distr['n'], distr['p']) - round(m.logpdf(i), 11)) < 10**-10, sep='\t')

    # ,,0.335898768556
    x, y, side = 800, 600, 'l'
    # print(x, y, -get_log_pvalue_new(x, y, side, distr), sep='\t')


    # for x,y in [(800, 600), (12, 5)]:
    # side = 'l' if x > y else 'r'
    # x, y, side = 12, 5, 'l'
    # print(x, y, -get_log_pvalue_new(x, y, side, distr), sep='\t')

    # print()

    S = 30
    for x in range(S + 1):
        y = S - x
        side = 'l' if x > y else 'r'
        cur = 10 ** (get_log_value(x, distr))
        cur2 = 10 ** (get_log_value(y, distr))
        # print(x, y, cur, cur2, 10**get_log_pvalue_new(x, y, side, distr), sep='\t')
        # print(cur, cur2, sep='\t')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
%filterVCF [options] <CONFIG> 

Filter VCF files by pipeline described in README.
<CONFIG> file must be a space separated list of:

<sample's name> <path to VCF>

@author: Manuel Allhoff

"""
from __future__ import print_function
#import pydevd;pydevd.settrace()
from optparse import OptionParser
import sys
from rgt.GenomicVariantSet import GenomicVariantSet
from rgt.GenomicRegionSet import GenomicRegionSet
from max_density import AlgGoldwasser
from itertools import combinations
from copy import copy

class HelpfulOptionParser(OptionParser):
    """An OptionParser that prints full help on errors."""
    def error(self, msg):
        self.print_help(sys.stderr)
        self.exit(2, "\n%s: error: %s\n" % (self.get_prog_name(), msg))

def load_data(path_sample_vcf):
    """Return list of GenomicVarientSets described by config file"""
    data = []
    with open(path_sample_vcf) as file:
        for line in file:
            tmp = line.split(" ")
            name, path = tmp[0], tmp[1]
            path = path.strip()
            data.append(GenomicVariantSet(path, name=name))
            
    return data

def print_length(l, info):
    """Print length of each GenomicVariant set in list l"""
    names = map(lambda x: x.name, l)
    print(info, file=sys.stderr)
    for i, name in enumerate(names):
        print(name, len(l[i]), file=sys.stderr, sep='\t')

def get_max_density(GenomicVariantSets, lowerBound=20, upperBound=50, max_it=5):
    """Compute max. density of homozygous SNPs and write results to stderr"""
    for GenomicVariantSet in GenomicVariantSets:
        max_pos = 0
        for record in GenomicVariantSet:
            if 'Mask' not in record.filter and not record.samples[0].is_het:
                if record.pos > max_pos:
                    max_pos = record.pos

    density_seq = [ [0, 1] for _ in range(max_pos + 2 * upperBound) ] 
    
    i=0
    for GenomicVariantSet in GenomicVariantSets:
        for record in GenomicVariantSet:
            if 'Mask' not in record.filter and not record.samples[0].is_het:
                density_seq[record.pos][0] += 1
                i += 1
        
    density_seq = map(lambda x: tuple(x), density_seq)
    
    print('Max. Density Iterations, lower bound: %s, upper bound: %s' %(lowerBound, upperBound), file=sys.stderr)
    print('(Regions to consider: %s)' %i, file=sys.stderr)
    
    for i in range(max_it):
        den, coord = AlgGoldwasser(density_seq, lowerBound, upperBound)
	s = sum(map(lambda x: x[0], density_seq[coord[0]:coord[1]]))
	print("%s. It.: density %s at %s - %s with %s SNPs" %(i+1, round(den, 5), coord[0], coord[1], s), file=sys.stderr)
        density_seq[coord[0]:coord[1]] = [(0,1)] * (coord[1] - coord[0])
        

def input():
    """Parse options"""
    parser = HelpfulOptionParser(usage=__doc__)
     
    parser.add_option("--t-mq", dest="t_mq", default=20, help="Threshold for mapping quality (MQ) [default: %default]")
    parser.add_option("--t-dp", dest="t_dp", default=20, help="Threshold for combined depth (DP) [default: %default]")
    parser.add_option("--dbSNP", dest="c_dbSNP", default=None, help="Check for dbSNP [default: %default]")
    parser.add_option("--list-WT", dest="list_wt", default=None, help="List of WildTypes [default: %default]")
    parser.add_option("--bed", dest="list_bed", default=None, help="Filter against BED file (e.g. TFBS) [default: %default]")
    parser.add_option("--max-density", dest="max_density", default=False, action="store_true", help="Perform max. density search [default: %default]")
    parser.add_option("--lowerBound", dest="lower_bound", default=15000, help="lower window bound for max. density search [default: %default]")
    parser.add_option("--upperBound", dest="upper_bound", default=30000, help="upper window bound for max. density search [default: %default]")
    
    (options, args) = parser.parse_args()
    
    vcf_list = args[0]
     
    i = 1
    if len(args) != i:
        parser.error("Exactly %s parameters are needed" %i)
        
    return options, vcf_list

def pipeline(sample_data, options):
    """Filter sample data by MQ, DP and dbSNP"""
    print_length(sample_data, "#unfiltered variants")
    
    #filter MQ
    for sample in sample_data:
        sample.filter('MQ', '>=', options.t_mq)
    
    print_length(sample_data, "#variants with MQ >= %s" %options.t_mq)
    
    #filter DP
    for sample in sample_data:
        sample.filter('DP', '>=', options.t_dp)
    
    print_length(sample_data, "#variants with DP >= %s" %options.t_dp)
    
    if options.c_dbSNP:
        for sample in sample_data:
            sample.filter_dbSNP()
    
        print_length(sample_data, "#variants after filtering by dbSNP")
    else:
        print("#Do not filter by dbSNP", file=sys.stderr)

def output_intersections(sample_data):
    """Compute and write VCF of all possible sample data subsets' intersection."""
    for i in range(2, len(sample_data)+1):
        for tuple in combinations(sample_data, i): 
        #tuples have size i and are all subsets of sample_data with size i
            for j, sample in enumerate(list(tuple)):
                if j == 0:
                    intersect_variants = GenomicVariantSet(name = 'intersection')
                    intersect_variants.reader = copy(sample.reader)
                    intersect_variants.sequences = copy(sample.sequences)
                else:
                    intersect_variants.intersect(sample)
            
            name = "-".join(map(lambda x: x.name, list(tuple)))
            print("Intersection:", name, len(intersect_variants), sep='\t', file=sys.stderr)  
            intersect_variants.write_vcf('intersect-%s.vcf' %name)

def main():
    options, vcf_list = input()
    
    #thres_mq = 20
    #thres_dp = 20 
    #filter_dbSNP = True
    #tfbs_motifs_path = '/home/manuel/workspace/cluster_p/human_genetics/exp/exp01_motifsearch_sox2/humangenetics_motifs/Match/chr11_mpbs.bed'
    
    sample_data = load_data(vcf_list)
    print("##Filter variants of samples", file=sys.stderr)
    pipeline(sample_data, options)
    
    if options.list_wt:
        wt_data = load_data(options.list_wt)
        print("##Filter variants of wildtypes", file=sys.stderr)
        pipeline(wt_data, options)
        union_wt = GenomicVariantSet(name = "union_wt")
        for wt in wt_data:
            union_wt.sequences += wt.sequences 
        
        print("#wildtype variants:", file=sys.stderr)
        print("union WT", len(union_wt), file=sys.stderr, sep="\t")
        
        #delete Wildtype
        for sample in sample_data:
            sample.subtract(union_wt)
        
        print_length(sample_data, "#variants after subtracting wildtypes")
    else:
        print("#Do not filter by wildtype", file=sys.stderr)
    
    if options.max_density:
        get_max_density(GenomicVariantSets=sample_data, lowerBound=options.lower_bound, upperBound=options.upper_bound)
    else:
        print("#Do not perform max. density search", file=sys.stderr)
        
    if options.list_bed:
        tfbs_motifs = GenomicRegionSet('tfbs_motifs')   
        tfbs_motifs.read_bed(options.list_bed)
        
        for sample in sample_data:
            sample.intersect(tfbs_motifs)
    
        print_length(sample_data, "#variants after filtering by BED file")
    else:
        print("#Do not filter by BED file", file=sys.stderr)
    
    print("#Compute intersection of sample's subsets (give intersection's name and size)")
    output_intersections(sample_data)
    
    print("#Write filtered sample files")
    for sample in sample_data:
        sample.write_vcf("%s-filtered.vcf" %sample.name)
    
if __name__ == '__main__':
    main()   
